from .whitelist import Whitelist

class URLWL(Whitelist):
    """IPv4 address whitelist.
    This class stores its whitelisted URLs  in the url table.
    """

    @staticmethod
    def getCreateTable():
        """Create the table to hold the URLs"""
        return "create table url (url char(256))"

    @staticmethod
    def storeDB(conn,urls):
        """Overwrite the URLs in the table with the specified URLs (string) in the database."""
        c=conn.cursor()
        vals=[]
        for url in urls:
            vals.append( (url,) )
        c.execute("delete from url")
        c.executemany("insert into url(url) values (?)",iter(vals))
        c.close()

    def isWhitelisted(self,conn,indicatorType,indicator):
        """Return whether or not the indicator of type indicatorType is whitelisted by this whitelist.
        This checks the indicator against the URLs in the table and requires an exact match to be considered whitelisted."""
        c=conn.cursor()
        c.execute("select url from url where url=?",(indicator,))
        rec=c.fetchone()
        c.close()
        return rec != None



import importlib
import io
import logging
import os
import sys

import toml

from lqmt.lqm.exceptions import ConfigurationError
from lqmt.lqm.logging import LQMLogging
from lqmt.lqm.sourcedir import DirectorySource
from lqmt.lqm.sourcefilter import SourceFilters
from lqmt.lqm.systemconfig import SystemConfig
from lqmt.lqm.tool import ToolChain
from lqmt.whitelist.master import MasterWhitelist
from lqmt.lqm.parsers.FlexTransform import parser as ft_parser
from lqmt.lqm.parsers.STIXParser import parser as stix_parser
from lqmt.lqm.parsers.RuleParser import parser as rule_parser

sys.path.append('tpl/toml')


class ToolInfo:
    """Holds info for future instantiation and instantiates tool instances
    """

    def __init__(self, toolClass, configClass):
        """Hold tool class object and config class object"""
        self._toolClass = toolClass
        self._configClass = configClass

    def create(self, configData, csvToolInfo, unhandledCSV):
        """Create a new config instance from the parameters passed in
        and then create a new tool instace using that configuration"""
        config = self._configClass(configData, csvToolInfo, unhandledCSV)
        tool = self._toolClass(config)
        return tool


class LQMToolConfig(object):
    """Holds the Configuration data for the LQMTool"""

    def __init__(self, configfile):
        self._logger = logging.getLogger("LQMT.Config")
        self._sources = []
        self._parsers = {}
        self._toolChains = {'pull': [], 'push': []}
        self._whitelist = None
        self._loggingCfg = None
        self._toolsList = []
        self._userConfig = {}
        self._pre_filters = []

        # load config files
        self._loadSystemConfig()
        self._loadUserConfig(configfile)
        # initialize logging
        self._initializeLogging()
        # process config files
        toolClasses = self._processSystemConfig()
        self._processUserConfig(toolClasses)
        if not self._sources and not self._toolChains['pull']:
            raise ConfigurationError("No sources specified")

    def _loadSystemConfig(self):
        """Load the System configuration file"""
        sysconf = SystemConfig()
        self._config = sysconf.getConfig()

    def _processSystemConfig(self):
        """
        Parses and initializes the system configuration
        :return: toolClasses, which is a dictionary holding classes for the tools assigned in the user configuration
        """
        self._initParserConfig(self._config['parsers'])
        toolClasses = self._initToolConfig(self._config)
        return toolClasses
    
    def _filter_parsers(self, parsers):
        """
        Filter parsers based on if they should be enabled or disabled. 
        :param parsers: dictionary containing parsers from system config
        :return: a dictionary of filtered parsers. 
        """
        enable = []
        disable = []
        if 'Parsers' in self._userConfig:
            parser_switch = self._userConfig['Parsers'].pop()
            if 'enable' in parser_switch:
                enable = parser_switch['enable']
            if 'disable' in parser_switch:
                disable = parser_switch['disable']
        
        for key, info in parsers.items():
            if key in disable:
                parsers[key]['default_enabled'] = False
            if key in enable:
                parsers[key]['default_enabled'] = True
        
        return parsers

    def _initParserConfig(self, parsers):
        """
        Initializes parsers using configuration details provided by the system configuration file
        :param parsers: Parser information passed from the system configuration file
        """

        parsers = self._filter_parsers(parsers)
        # path info is loaded from the config files
        for key, parserinfo in parsers.items():

            # gets parser class from flexT
            if parserinfo['parser_class'] == 'STIXParser':
                parserClass = getattr(stix_parser, parserinfo['parser_class'])
            elif parserinfo['parser_class'] == 'RuleParser':
                parserClass = getattr(rule_parser, parserinfo['parser_class'])
            else:  # gets parser class from flexT, all others use FlexT
                parserClass = getattr(ft_parser, parserinfo['parser_class'])

            # if particular configs were defined for the parser, pass them to FlexT parser and append to self._parsers
            if parserinfo['default_enabled'] is True:
                if parserinfo['configs']:
                    if type(parserinfo['format']) is list:
                        for p_format in parserinfo['format']:
                            self._parsers[p_format] = parserClass(parserinfo['configs'])
                    else:
                        self._parsers[parserinfo['format']] = parserClass(parserinfo['configs'])
                else:
                    self._parsers[parserinfo['format']] = parserClass()

        self._logger.debug("Parsers loaded: %s" % ', '.join(self._parsers.keys()))

    def _initToolConfig(self, config):
        """
        Set up the system path for the configured tools and create the configuration data for those tools so they can
        be created later.
        """

        # Use user configuration to only load tools the user specifies. The CSV tool is appended at the end because
        # it is needed later regardless of what tools are loaded.
        tooldefs = config['tools']
        usertooldefs = []

        for key, config in self._userConfig['Tools'].items():
            usertooldefs.append(key)

        # usertooldefs = self._userConfig['Tools'].keys()
        # usertooldefs = list(usertooldefs)
        usertooldefs.append('CSV')

        # uses dictionary comprehension to filter out only user specified tools from the tooldefs dictionary
        tooldefs = {tool_key: tooldefs[tool_key] for tool_key in usertooldefs}

        toolClasses = {}
        for key in list(tooldefs.keys()):
            toolinfo = tooldefs[key]
            if 'additional_paths' in toolinfo:
                # add any additional paths needed by the tool
                add_path = toolinfo['additional_paths']
                sys.path.extend(add_path)
            # import the tool & config modules
            mod = importlib.import_module("lqmt.tools." + toolinfo['module'] + ".tool")
            toolClass = getattr(mod, toolinfo['tool_class'])
            mod = importlib.import_module("lqmt.tools." + toolinfo['module'] + ".config")
            # get the class "object" for later creation
            cfgClass = getattr(mod, toolinfo['config_class'])
            toolClasses[key] = ToolInfo(toolClass, cfgClass)

        self._logger.debug("Loading the following tools: %s" % ', '.join(tooldefs))
        return toolClasses

    def _loadUserConfig(self, configFile):
        """
        Loads the user configuration file
        :param configFile: String that defines the path of the user configuration file.
        """
        # if provided config is an existing directory, open and read the file. Else it is assumed to be a config string
        if os.path.exists(configFile):
            cfg = open(configFile)
            topLevelConfig = toml.loads(cfg.read())
            cfg.close()
        else:
            configFile = io.StringIO(configFile)
            topLevelConfig = toml.load(configFile)

        self._userConfig.update(topLevelConfig)

    def _initializeLogging(self):
        if 'Logging' in self._userConfig:
            self._loggingCfg = LQMLogging(self._userConfig['Logging'])

    def _processUserConfig(self, toolClasses):
        """Process the user-level configuration"""

        # Add any sources specified
        self._addSourcesConfig(self._userConfig)

        # If a whitelist was specified, create it
        if 'Whitelist' in self._userConfig:
            self._whitelist = MasterWhitelist(configData=self._userConfig['Whitelist'])

        # create any tools and tool chains in the top-level user config file
        tools = self._createTools(self._userConfig, toolClasses)
        self._createToolChains(self._userConfig, tools)

    def _createTools(self, config, toolClasses):
        if 'Tools' not in config:
            return
        tools = config['Tools']

        theseTools = {'pull': {}, 'push': {}}
        for key in tools:
            if key in toolClasses:
                toolInfo = toolClasses[key]
                self._toolsList.append(key)
                for cfgData in tools[key]:
                    tool = toolInfo.create(cfgData, toolClasses['CSV'], self._config['UnprocessedCSV'])
                    tool.toolName = key
                    if 'accepted_formats' in self._config['tools'][key]:
                        tool.dataFormat = self._config['tools'][key]['accepted_formats']
                    theseTools[self.get_tool_type(key)][tool.getName()] = tool

        return theseTools

    @staticmethod
    def get_tool_type(toolname):
        tool_type = 'push'
        if 'pull' in toolname.lower() or 'from' in toolname.lower():  # TODO: Added from, a variable might be better than name contents
            tool_type = 'pull'

        return tool_type

    def _createToolChains(self, config, localTools, globalTools=None):
        if 'ToolChains' not in config:
            return

        chains = config['ToolChains']

        for chainCfg in chains:
            if 'active' in chainCfg and chainCfg['active'] is True:

                if localTools['push']:
                    self._toolChains['push'].append(
                        self._createToolChain(chainCfg, localTools['push'], globalTools))

                if localTools['pull']:
                    self._toolChains['pull'].append(
                        self._createToolChain(chainCfg, localTools['pull'], globalTools))
            else:
                self._logger.info("Toolchain {0} is currently set as inactive in the user configuration. "
                                  "Tools in this toolchain will not run.".format(chainCfg['name']))

    def _createToolChain(self, chainCfg, localTools, globalTools):
        chain = []
        allEnabled = True

        for toolName in chainCfg['chain']:
            tool = None
            if toolName in localTools:
                tool = localTools[toolName]
            elif globalTools is not None and toolName in globalTools:
                tool = globalTools[toolName]
            else:
                pass
            # raise ConfigurationError("Named tool not found: " + toolName)
            if tool is not None:
                chain.append(tool)
                allEnabled = allEnabled and tool.isEnabled()
                if not tool.isEnabled():
                    self._logger.error("Tool chain {0} is disabled due to tool {1} being disabled".format(chainCfg['name'],
                                                                                                      tool.getName()))

        chain = ToolChain(chain, chainCfg['name'], enabled=allEnabled)
        if allEnabled:
            self._logger.info("Created tool chain: {0}".format(chain.getName()))
        chain.printTools()
        return chain

    def _addSourcesConfig(self, config):
        if 'Source' not in config:
            return
        srcCfgs = config["Source"]

        for key in srcCfgs:
            for cfg in srcCfgs[key]:
                if key == "Directory":
                    self._sources.append(DirectorySource(cfg))
                elif key == "Filters":
                    self._pre_filters = SourceFilters(cfg)

    def getSources(self):
        return self._sources

    def getParser(self, fmt):
        if fmt not in self._parsers:
            return None
        return self._parsers[fmt]

    def getToolChains(self):
        return self._toolChains

    def getWhitelist(self):
        return self._whitelist

    def getToolsList(self):
        """
        :return: Returns a list of tools that are currently active in the toolchain.
        """
        return self._toolsList

    def getSourceFilters(self):
        return self._pre_filters



class SystemConfig:
    def __init__(self):
        self.system_config = {
            'tools': {
                'Checkpoint': {
                    'module': 'to_checkpoint', 'config_class': 'CheckpointConfig', 'tool_class': 'ToCheckpoint',
                    'accepted_formats': ['Alert']
                },
                'PaloAlto': {
                    'module': 'to_paloalto', 'config_class': 'PaloAltoConfig', 'tool_class': 'ToPaloAlto',
                    'additional_paths': ['tpl'], 'accepted_formats': ['Alert']
                },
                'CSV': {
                    'module': 'to_csv', 'config_class': 'CSVConfig', 'tool_class': 'ToCSV',
                    'accepted_formats': ['Alert']
                },
                'CEF': {
                    'module': 'to_cef', 'config_class': 'CEFConfig', 'tool_class': 'ToCEF',
                    'accepted_formats': ['Alert']
                },
                'ArcSight': {
                    'module': 'to_arcsight', 'config_class': 'ArcSightConfig', 'tool_class': 'ToArcSight',
                    'accepted_formats': ['Alert']
                },
                'SysLog': {
                    'module': 'to_syslog', 'config_class': 'SysLogConfig', 'tool_class': 'ToSysLog',
                    'accepted_formats': ['Alert']
                },
                'FlexText': {
                    'module': 'to_flextext', 'config_class': 'FlexTextConfig', 'tool_class': 'ToFlexText',
                    'accepted_formats': ['Alert']
                },
                'Splunk': {
                    'module': 'to_splunk', 'config_class': 'SplunkConfig', 'tool_class': 'ToSplunk',
                    'accepted_formats': ['Alert']
                },
                'Bro': {
                    'module': 'to_bro', 'config_class': 'BroConfig', 'tool_class': 'ToBro',
                    'accepted_formats': ['Alert']
                },
                'MBL': {
                    'module': 'to_mbl', 'config_class': 'MBLConfig', 'tool_class': 'ToMBL',
                    'accepted_formats': ['Alert']
                },
                'From_MBL': {
                    'module': 'from_mbl', 'config_class': 'FromMBLConfig', 'tool_class': 'FromMBL',
                    'accepted_formats': ['Alert']
                },
                'Pull_Test': {
                    'module': 'pull_test', 'config_class': 'PullTestConfig', 'tool_class': 'PullTest',
                    'accepted_formats': ['Alert']
                },
                'Snort': {
                    'module': 'to_snort', 'config_class': 'SnortConfig', 'tool_class': 'ToSnort',
                    'accepted_formats': ['StixFile', 'RuleFile']
                },
                'From_Snort': {
                    'module': 'from_snort', 'config_class': 'FromSnortConfig', 'tool_class': 'FromSnort',
                    'accepted_formats': ['StixFile']
                }
            },
            'UnprocessedCSV': {
                'name': 'for-unprocessed',
                'fields': ['dataItemID', 'fileID', 'detectedTime', 'reportedTime', 'processedTime', 'indicator',
                           'indicatorType', 'indicatorDirection', 'secondaryIndicator', 'secondaryIndicatorType',
                           'secondaryIndicatorDirection', 'directSource', 'secondarySource', 'action1', 'duration1',
                           'action2', 'duration2', 'reason1', 'reference1', 'reason2', 'reference2', 'majorTags',
                           'minorTags', 'restriction', 'sensitivity', 'reconAllowed', 'priors', 'confidence',
                           'severity', 'relevancy', 'relatedID', 'relationType', 'comment', 'fileHasMore']
            },
            'parsers': {
                'Cfm20Alert': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'Cfm20Alert': 'resources/sampleConfigurations/cfm20alert.cfg'
                    },
                    'format': 'Cfm20Alert',
                    'default_enabled': True
                },
                'Cfm13Alert': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'Cfm13Alert': 'resources/sampleConfigurations/cfm13.cfg'
                    },
                    'format': 'Cfm13Alert',
                    'default_enabled': True
                },
                'stixtlp': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'stix-tlp': 'resources/sampleConfigurations/stix_tlp.cfg',
                        'STIX': 'resources/sampleConfigurations/stix_tlp.cfg'

                    },
                    'format': ['stix-tlp', 'STIX'],
                    'default_enabled': True
                },
                'MBL': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'mbl': 'resources/sampleConfigurations/MBL.cfg',
                        'Cfm20Alert': 'resources/sampleConfigurations/cfm20alert.cfg',
                        'stix-tlp': 'resources/sampleConfigurations/stix_tlp.cfg'

                    },
                    'format': 'mbl',
                    'default_enabled': True
                },
                'IIDactiveBadHosts': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'IIDactiveBadHosts': 'resources/sampleConfigurations/iid_host_active.cfg'
                    },
                    'format': 'IIDactiveBadHosts',
                    'default_enabled': False
                },
                'IIDcombinedURL': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'IIDcombinedURL': 'resources/sampleConfigurations/iid_combined_recent.cfg'
                    },
                    'format': 'IIDcombinedURL',
                    'default_enabled': False
                },
                'IIDdynamicBadHosts': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'IIDdynamicBadHosts': 'resources/sampleConfigurations/iid_host_dynamic.cfg'
                    },
                    'format': 'IIDdynamicBadHosts',
                    'default_enabled': False
                },
                'IIDrecentBadIP': {
                    'module': 'lqmt.lqm.parsers.FlexTransform',
                    'parser_class': 'FlexTransformParser',
                    'configs': {
                        'LQMTools': 'resources/sampleConfigurations/lqmtools.cfg',
                        'IIDrecentBadIP': 'resources/sampleConfigurations/iid_ipv4_recent.cfg'
                    },
                    'format': 'IIDrecentBadIP',
                    'default_enabled': False
                },
                'STIXParser': {
                    'module': 'lqmt.lqm.parsers.STIXParser',
                    'parser_class': 'STIXParser',
                    'configs': {
                        'STIXparserConfig': 'resources/parser_configs/stixparser.toml'
                    },
                    'format': ['stix-tlp', 'STIX'],  # OtherFormat
                    'default_enabled': False
                },
                'RuleParser': {
                    'module': 'lqmt.lqm.parsers.RuleParser',
                    'parser_class': 'RuleParser',
                    'configs': {
                        'RuleParserConfig': 'resources/parser_configs/ruleparser.toml'
                    },
                    'format': ['SnortRules', 'YaraRules'],  # OtherFormat, SnortJson, AnyFormat
                    'default_enabled': False
                }
            }
        }

    def getConfig(self):
        return self.system_config



import logging
import requests
from lqmt.lqm.tool import Tool
from lqmt.lqm.data import AlertAction
from lqmt.tools.to_splunk.splunk_api import ApiHandler, create_message
from requests.packages.urllib3.exceptions import InsecureRequestWarning


class ToSplunk(Tool):
    def __init__(self, config):
        """
        ToSplunk tool. Used to push data to Splunk using Splunk web API.

        :param config: configuration file
        """
        super().__init__(config, [AlertAction.get('All')])
        self._logger = logging.getLogger("LQMT.ToSplunk.{0}".format(self.getName()))
        self._splunk_token = ""

        if self._config.cert_check is False:
            requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

        self.handler = ApiHandler(
            self._config.host,
            self._config.port,
            self._config.username,
            self._config.password,
            cert_check=self._config.cert_check,
            source=self._config.source,
            sourcetype=self._config.sourcetype,
            index=self._config.index
        )

    def initialize(self):
        super().initialize()

    def process(self, alert):
        """
        Process function. Handles the processing of data for the tool. 
        """
        self.handler.send_message(create_message(alert))

    def commit(self):
        pass

    def cleanup(self):
        self.handler.__exit__()



import arrow
from lqmt.lqm.exceptions import ConfigurationError

class SourceFilters(object):
    def __init__(self, config):
        """
        Initialization function for class handling file source filters
        :param config: Dictionary of the user configuration section for Sources.Filters
        :return: None
        """
        self._site_includes = []
        self._site_excludes = []
        self._payload_types = []
        self._payload_formats = []
        self._sensitivities = []
        self._restrictions = []
        self._reconnaissance = []
        self._max_file_age = None

        if 'site_includes' in config:
            self._site_includes = self.__list_lower(config['site_includes'])
        if 'site_excludes' in config:
            self._site_excludes = self.__list_lower(config['site_excludes'])
        if 'payload_types' in config:
            self._payload_types = self.__list_lower(config['payload_types'])
        if 'payload_formats' in config:
            self._payload_formats = self.__list_lower(config['payload_formats'])
        if 'sensitivities' in config:
            self._sensitivities = self.__list_lower(config['sensitivities'])
        if 'restrictions' in config:
            self._restrictions = self.__list_lower(config['restrictions'])
        if 'reconnaissance' in config:
            self._reconnaissance = self.__list_lower(config['reconnaissance'])
        if 'max_file_age' in config:
            self._max_file_age = config['max_file_age']

    def __list_lower(self, value):
        """
        Lower cases a list that is provided.
        :param value: List of strings to be converted to lower case
        :return ret: Converted list
        """
        ret = []

        for i in value:
            ret.append(i.lower())

        return ret

    def checkAllFilters(self, metafile):
        """
        Performs all the available filter checks against the file meta-data.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        if not self.checkSendingSite(metafile):
            return False
        if not self.checkPayloadType(metafile):
            return False
        if not self.checkPayloadFormat(metafile):
            return False
        if not self.checkDataSensitivity(metafile):
            return False
        if not self.checkSharingRestrictions(metafile):
            return False
        if not self.checkReconPolicy(metafile):
            return False
        if not self.checkFileAge(metafile):
            return False

        return True

    def checkSendingSite(self, metafile):
        """
        Checks the Includes and Excludes list against the sending site in the meta data.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False

        if metafile is None:
            return False

        # Check the Sending Sites
        if 'SendingSite' in metafile:
            if metafile['SendingSite'].lower() in self._site_includes or not self._site_includes:
                ret = True
            # if in both includes and excludes, will filter out the file
            if metafile['SendingSite'].lower() in self._site_excludes and self._site_excludes:
                ret = False

        return ret

    def checkPayloadType(self, metafile):
        """
        Checks the Payload Type against configured list.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False

        if metafile is None:
            return False

        if not self._payload_types:
            return True

        # Check the PayloadType
        if 'PayloadType' in metafile:
            if metafile['PayloadType'].lower() in self._payload_types:
                ret = True

        return ret

    def checkPayloadFormat(self, metafile):
        """
        Checks the Payload Format against configured list.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False

        if metafile is None:
            return False

        if not self._payload_formats:
            return True

        # Check the PayloadFormat
        if 'PayloadFormat' in metafile:
            if metafile['PayloadFormat'].lower() in self._payload_formats:
                ret = True

        return ret

    def checkDataSensitivity(self,
                             metafile):
        """
        Checks the Data Sensitivity against configured list.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False

        if metafile is None:
            return False

        if not self._sensitivities:
            return True

        # Check the DataSensitivity
        if 'DataSensitivity' in metafile:
            if metafile['DataSensitivity'].lower() in self._sensitivities:
                ret = True

        return ret

    def checkSharingRestrictions(self, metafile):
        """
        Checks the Sharing Restrictions against configured list.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False

        if metafile is None:
            return False

        if not self._restrictions:
            return True

        # Check the SharingRestrictions
        if 'SharingRestrictions' in metafile:
            if metafile['SharingRestrictions'].lower() in self._restrictions:
                ret = True

        return ret

    def checkReconPolicy(self, metafile):
        """
        Checks the Reconnaissance Policy against configured list.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False

        if metafile is None:
            return False

        if not self._reconnaissance:
            return True

        # Check the ReconPolicy
        if 'ReconPolicy' in metafile:
            if metafile['ReconPolicy'].lower() in self._reconnaissance:
                ret = True

        return ret

    def checkFileAge(self, metafile):
        """
        Checks the Sent Timestamp against a shifted date/time for maximum file age.
        :param metafile: Meta-data file to compare to the filter parameters
        :return: Boolean for whether file has passed the pre-filtering settings
        """
        ret = False
        now = arrow.utcnow()
        filetime = None
        num = 0
        second = ['s', 'sec', 'secs', 'second', 'seconds']
        minute = ['m', 'min', 'minute', 'minutes']
        hour = ['h', 'hr', 'hrs', 'hour', 'hours']
        day = ['d', 'day', 'days']
        week = ['w', 'week', 'weeks']
        month = ['mon', 'month', 'months']
        year = ['y', 'yr', 'yrs', 'year', 'years']

        if metafile is None:
            return False

        if self._max_file_age is None:
            return True

        cols = self._max_file_age.split(' ')
        if cols[0].isdigit():
            num = -1 * int(cols[0])
        else:
            raise ConfigurationError(
                "Unable to parse file age \"{0}\" expecting %d %s format (e.g. 2 weeks)".format(self._max_file_age))

        if cols[1] in second:
            shift = now.shift(seconds=num)
        elif cols[1] in minute:
            shift = now.shift(minutes=num)
        elif cols[1] in hour:
            shift = now.shift(hours=num)
        elif cols[1] in day:
            shift = now.shift(days=num)
        elif cols[1] in week:
            shift = now.shift(weeks=num)
        elif cols[1] in month:
            shift = now.shift(months=num)
        elif cols[1] in year:
            shift = now.shift(years=num)
        else:
            raise ConfigurationError(
                "Unable to determine time string in \"{0}\". Refer to documentation for accepted time strings.".format(self._max_file_age))

        if 'SentTimestamp' in metafile:
            filetime = arrow.get(metafile['SentTimestamp'])
            if filetime > shift:
                ret = True

        return ret



from lqmt.lqm.tool import ToolConfig
from lqmt.lqm.exceptions import ConfigurationError
from lqmt.lqm.data import Alert
import logging
import os
import datetime


class CSVConfig(ToolConfig):
    def __init__(self, configData, csvToolInfo, unhandledCSV):
        super().__init__(configData, csvToolInfo, unhandledCSV)
        self.logger = logging.getLogger("LQMT.CSV.{0}".format(self.getName()))

        self._file = self.validation('file', str, required=True)
        self.fields = self.validation('fields', list, required=True)

        # Additional checks:
        invalid = self._initFields(self.fields)
        if invalid:
            raise ConfigurationError("Invalid alert field(s) provided. Invalid field(s): {0}".format(','.join(invalid)))

        # File incrementing. Make this an option rather then a default. See to_bro or to_flextext tools
        base, extension = os.path.splitext(self._file)
        file_name = "." + datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
        self._file = base + file_name + extension

    def getFile(self):
        return self._file

    def _initFields(self, fields):
        invalid = []
        for fld in fields:
            if not Alert.isValidField(fld):
                invalid.append(fld)

        return invalid

    def getFields(self):
        return self.fields



"""
Created on March 29, 2018

@author: grjohnson
"""
import os
import logging
import toml
from lqmt.lqm.data import RuleFile


class RuleParser(object):
    def __init__(self, config=None):
        """
        :param config: configuration file
        """
        # currently no config parameters
        self._logger = logging.getLogger("LQMT.Parsers")

        # accepted configuration values
        self._accepted_rules = ['snort', 'yara']

        # default lists
        self._sources = []
        self._rules = []
        self._line_offset = 0  # allows for non-snort compliant header that can be removed

        if config is not None:
            # parse config files
            for p, c in config.items():
                self.__parse_config(c)

    def __parse_config(self, configfile):
        """
        Extracts the defined configuration parameters for the parser.
        Important pre-condition is the user must configure elements and rules against a predefined enumeration.

        :param configfile: configuration file
        :return None:
        """
        if os.path.exists(configfile):
            cfg = open(configfile)
            topconfig = toml.loads(cfg.read())
            cfg.close()

            if 'Filters' in topconfig:
                if 'sources' in topconfig['Filters'][0]:
                    self._sources = topconfig['Filters'][0]['sources']

                if 'rules' in topconfig['Filters'][0]:
                    temp = topconfig['Filters'][0]['rules']
                    for x in temp:
                        if x.lower() in self._accepted_rules:
                            self._rules.append(x.lower())
                        else:
                            self._logger.warning("Incompatible rule for parser " + x)

                if 'start_offset' in topconfig['Filters'][0]:
                    self._line_offset = topconfig['Filters'][0]['start_offset']


    def __check_meta(self, meta):
        """
        Parses the Meta Data file looking for originators and custodians that match the user requested information source.

        :param meta: Meta data envelope for the file to be parsed
        :return Boolean: True if match is found, otherwise False.
        """
        ret = False

        if not self._sources:
            return True

        # If there is no entry returns false for finding a match
        if 'SendingSite' in meta:
            l = meta['SendingSite']
            if l in self._sources:
                return True

        if 'DownloadElementExtendedAttribute' in meta:
            l = meta['DownloadElementExtendedAttribute']
            # this element can be either a list of dictionaries or a dictionary
            if type(l) is dict:
                if l['Field'] == 'Originator' or l['Field'] == 'Custodian':
                    if l['Value'] in self._sources:
                        return True
            elif type(l) is list:
                for x in l:
                    if x['Field'] == 'Originator' or x['Field'] == 'Custodian':
                        if x['Value'] in self._sources:
                            return True
            else:
                ret = False

        if ret is False:
            self._logger.info("Did not identify the requested source in meta-data file.")

        return ret

    def __check_filetype(self, meta):
        ret = 'unknown'

        if 'PayloadFormat' in meta:
            l = meta['PayloadFormat']
            if l == 'SnortRules':
                ret = 'snort'
            elif l == 'YaraRules':
                ret = 'yara'

        return ret

    def custom_parser(self, datafile):
        ret = []

        return ret

    def parse(self, datafile, meta=None):
        alerts = []
        source_match = True
        rule_type = 'unknown'
        rules_dict = {'snort': [], 'yara': []}

        try:
            alert = RuleFile()

            if meta:
                # filter by sources
                source_match = self.__check_meta(meta)
                # determine the file format - snort versus yara
                rule_type = self.__check_filetype(meta)

            if source_match:
                if os.path.exists(datafile):
                    with open(datafile, 'r') as file:
                        data = file.read()
                        alert.setRawFile(data)

                if rule_type == 'snort' and 'snort' in self._rules:
                    with open(datafile, 'r') as file:
                        data_l = file.readlines()

                    # seek to start line (array of start line by sources?)
                    rules_dict['snort'] = data_l[self._line_offset:]
                    alert.setRules(rules_dict)
                elif rule_type == 'yara' and 'yara' in self._rules:
                    rules_dict['yara'].append(data)
                    alert.setRules(rules_dict)
                else:
                    self._logger.warning(
                        "LQMT-Rule-Parser: Unrecognized or unconfigured rule type={0}".format(rule_type))

                alerts.append(alert)
        except Exception as e:
            self._logger.error("LQMT-Rule-Parser: Problem with parsed data. Exception={0}".format(e))

        return alerts



class UnprocessedAlertHandler(object):
    """Object to handle unprocessed alerts."""

    def __init__(self, csvTool):
        self._toCSV = csvTool

    def initialize(self):
        self._toCSV.initialize()

    def unprocessed(self, alert):
        self._toCSV.process(alert)



from unittest import TestCase, main
from lqmt.lqm.parsers.FlexTransform.parser import FlexTransformParser
from lqmt.lqm.systemconfig import SystemConfig
from lqmt.test.test_data.sample_inputs import CFM13ALERT, CFM20ALERT, STIX
import io
import time
import uuid


class TestParser(TestCase):
    """
    Testing class for the parser alert parser.
    """

    def setUp(self):
        sysconf = SystemConfig()
        self.sys_config = sysconf.getConfig()
        self.sys_config = self.sys_config['parsers']
        self.flext = FlexTransformParser()

        # Add parsers for the different data formats
        self.flext.add_parser('LQMTools', 'resources/sampleConfigurations/lqmtools.cfg')
        self.flext.add_parser('Cfm13Alert', 'resources/sampleConfigurations/cfm13.cfg')
        self.flext.add_parser('Cfm20Alert', 'resources/sampleConfigurations/cfm20alert.cfg')
        self.flext.add_parser('stixtlp', 'resources/sampleConfigurations/stix_tlp.cfg')
        self.flext.add_parser('STIX', 'resources/sampleConfigurations/stix_tlp.cfg')

        # Standard timestamp that can be used so all meta files use the same time and for easier validation
        self.time = str(time.time()).split('.')[0]

        # Parse all the data ahead of the tests
        self.cfm13_parsed_data = self.flext.parse(io.StringIO(CFM13ALERT), self.getMeta("Cfm13Alert"))
        self.cfm20_parsed_data = self.flext.parse(io.StringIO(CFM20ALERT), self.getMeta("Cfm20Alert"))
        self.stix_parsed_data = self.flext.parse(io.StringIO(STIX), self.getMeta("STIX"))

    def getMeta(self, payloadFormat):
        """
        Method for generating metadata. Relatively static for now, but can be expanded to be randomized on each test run

        :param payloadFormat: Expected format of the payload.
        :return: Returns metadata in a dictionary
        """
        meta = {
            "PayloadFormat": payloadFormat,
            "SendingSite": "ANL",
            "PayloadType": "Alert",
            "UploadID": str(uuid.uuid4()).upper(),
            "FileName": "TestAlert",
            "SentTimestamp": self.time,
        }

        return meta

    # CFM13 format tests
    def test_cfm13_content_returned(self):
        self.assertEquals(len(self.cfm13_parsed_data), 1)

    def test_cfm13_indicator(self):
        self.assertEquals(self.cfm13_parsed_data[0]._indicator, "10.10.10.10")

    def test_cfm13_indicator_type(self):
        self.assertEquals(self.cfm13_parsed_data[0]._indicatorType, "IPv4Address")

    def test_cfm13_action(self):
        self.assertEquals(self.cfm13_parsed_data[0]._action1, "Block")

    def test_cfm13_duration(self):
        self.assertEquals(self.cfm13_parsed_data[0]._duration1, "86400")
        self.assertIsNone(self.cfm13_parsed_data[0]._duration2)

    def test_cfm13_sensitivity(self):
        self.assertEquals(self.cfm13_parsed_data[0]._sensitivity, "noSensitivity")

    def test_cfm13_restriction(self):
        self.assertEquals(self.cfm13_parsed_data[0]._restriction, "AMBER")

    # CFM20 format tests
    def test_cfm20_content_returned(self):
        self.assertEquals(len(self.cfm20_parsed_data), 1)

    def test_cfm20_indicator(self):
        self.assertEquals(self.cfm20_parsed_data[0]._indicator, "8675:a289:5:102c::bd8:baac")

    def test_cfm20_indicator_type(self):
        self.assertEquals(self.cfm20_parsed_data[0]._indicatorType, "IPv6Address")

    def test_cfm20_action(self):
        self.assertEquals(self.cfm13_parsed_data[0]._action1, "Block")

    def test_cfm20_duration(self):
        self.assertEquals(self.cfm20_parsed_data[0]._duration1, "86400")
        self.assertIsNone(self.cfm20_parsed_data[0]._duration2)

    def test_cfm20_sensitivity(self):
        self.assertEquals(self.cfm20_parsed_data[0]._sensitivity, "noSensitivity")

    def test_cfm20_restriction(self):
        self.assertIsNone(self.cfm20_parsed_data[0]._restriction)

    # STIX format tests
    def test_stix_content_returned(self):
        self.assertEquals(len(self.stix_parsed_data), 11)

    def test_stix_indicator(self):
        self.assertEquals(self.stix_parsed_data[1]._indicator, "13.13.13.13")
        self.assertEquals(self.stix_parsed_data[9]._indicator, "bad.domain.be/poor/path")

    def test_stix_indicator_type(self):
        self.assertEquals(self.stix_parsed_data[1]._indicatorType, "IPv4Address")
        self.assertEquals(self.stix_parsed_data[5]._indicatorType, "FilePath")

    def test_stix_action(self):
        self.assertEquals(self.stix_parsed_data[1]._action1, "Block")

    def test_stix_duration(self):
        self.assertEquals(self.stix_parsed_data[1]._duration1, "86400")
        self.assertIsNone(self.stix_parsed_data[1]._duration2)

    def test_stix_sensitivity(self):
        self.assertEquals(self.stix_parsed_data[1]._sensitivity, "noSensitivity")

    def test_stix_restriction(self):
        self.assertIsNone(self.stix_parsed_data[1]._restriction)

if __name__ == '__main__':
    main()



import os
import logging
import datetime
from lqmt.lqm.tool import ToolConfig


class BroConfig(ToolConfig):
    def __init__(self, configData, csvToolInfo, unhandledCSV):
        super().__init__(configData, csvToolInfo, unhandledCSV)
        self.logger = logging.getLogger("LQMT.ToBro.{0}".format(self.getName()))
        self.header_fields = self.validation('header_fields', list, default="all")
        self.file = self.validation('file', str, default="./lqmt-bro-feed.txt")
        self.increment_file = self.validation('increment_file', bool, default=False)
        self.null_value = self.validation('null_value', str, default='-')

        if self.increment_file:
            base, extension = os.path.splitext(self.file)
            file_name = "."+datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
            self.file = base + file_name + extension



# from enum import Enum
import logging
from .ipv4wl import IPv4WL
from .ipv6wl import IPv6WL
from .ipv4snwl import IPv4SubnetWL
from .ipv6snwl import IPv6SubnetWL
from .domainwl import DomainWL
from .hostwl import HostWL
from .urlwl import URLWL
import sys
import os
import hashlib
import io

import sqlite3
import toml

sys.path.append('tpl/toml')


class IndicatorTypes(object):
    """Enumeration of possible indicator types
    """
    ipv4 = 'ipv4'
    ipv6 = 'ipv6'
    ipv4subnet = 'ipv4wl'
    ipv6subnet = 'ipv6wl'
    domain = 'domain'
    host = 'host'
    url = 'url'
    filepath = "filepath"
    filehash = "filehash"


class MasterWhitelist(object):
    """The master whitelist object.
    This class is responsible for maintaining the database of all the whitelists
    and it is the central point for checking for whitelisted indicators
    """

    def __init__(self, configData=None, configFile=None, configStr=None):
        self._logger = logging.getLogger("LQMT.Whitelist")
        if configData is not None:
            self._loadConfig(configData)
        elif configFile is not None:
            self._loadConfigFromFile(configFile)
        elif configStr is not None:
            self._loadConfigFromStr(configStr)

        self._updateDB()
        self._whitelists = {IndicatorTypes.ipv4: IPv4WL(), IndicatorTypes.ipv6: IPv6WL(),
                            IndicatorTypes.ipv4subnet: IPv4SubnetWL(), IndicatorTypes.ipv6subnet: IPv6SubnetWL(),
                            IndicatorTypes.domain: DomainWL(), IndicatorTypes.host: HostWL(),
                            IndicatorTypes.url: URLWL()}
        self._initMapping()

    def _initMapping(self):
        """Create the mapping of indicator types to the types of whitelists they will be checked against"""
        self._indicatorMapping = {
            IndicatorTypes.ipv4: [IndicatorTypes.ipv4, IndicatorTypes.ipv4subnet],
            IndicatorTypes.ipv4subnet: [IndicatorTypes.ipv4subnet],
            IndicatorTypes.ipv6: [IndicatorTypes.ipv6, IndicatorTypes.ipv6subnet],
            IndicatorTypes.ipv6subnet: [IndicatorTypes.ipv6subnet],
            IndicatorTypes.domain: [IndicatorTypes.domain],
            IndicatorTypes.host: [IndicatorTypes.host, IndicatorTypes.domain],
            IndicatorTypes.url: [IndicatorTypes.url]
        }

    def _loadConfigFromFile(self, configFile):
        cfg = open(configFile)
        config = toml.loads(cfg.read())
        cfg.close()
        self._loadConfig(config['Whitelist'])

    def _loadConfigFromStr(self, configStr):
        config_str = io.StringIO(configStr)
        config = toml.load(config_str)
        self._loadConfig(config['Whitelist'])

    def _loadConfig(self, configData):
        self.db = configData['dbfile']
        self.whitelistFile = configData['whitelist']

    def _updateDB(self):
        """
        Updates the whitelist database if it detects a change in the whitelist.txt file.
        """
        if not os.path.exists(self.db):
            # Create it if not already there
            self._createDB()
        self.conn = sqlite3.connect(self.db)
        connection = self.conn.cursor()
        # compute the md5 hash of the whitelist text file
        md5_hash = hashlib.md5()
        wlf = open(self.whitelistFile, "rb")
        md5_hash.update(wlf.read())
        md5 = md5_hash.hexdigest()
        # and compare it against the md5 of the last file loaded
        connection.execute("select md5 from md5")
        rec = connection.fetchone()
        connection.close()
        if rec is None or rec[0] != md5:
            # if it is not the same, then reload the whitelist file
            self._reloadDB(md5)

    def _reloadDB(self, md5):
        """
        Reloads all database tables with new values from the whitelist.txt file
        :param md5: used to update the md5 table with the newly calculated md5
        """
        whitelist = self._loadWhitelistFile()
        DomainWL.storeDB(self.conn, whitelist['Domain'])
        HostWL.storeDB(self.conn, whitelist['Host'])
        IPv4WL.storeDB(self.conn, whitelist['IPv4Address'])
        IPv4SubnetWL.storeDB(self.conn, whitelist['IPv4Subnet'])
        IPv6WL.storeDB(self.conn, whitelist['IPv6Address'])
        # IPv6SubnetWL.storeDB(self.conn,whitelist['IPv6Subnet'])
        URLWL.storeDB(self.conn, whitelist['URL'])
        connection = self.conn.cursor()
        connection.execute("delete from md5")
        connection.execute("insert into md5 values (?)", (md5,))
        self.conn.commit()
        connection.close()

    def _loadWhitelistFile(self):
        """
        Loads data from the whitelist file.
        :return: returns an dict containing the values from the whitelist.txt file
        """
        wlf = open(self.whitelistFile, "r")
        whitelist = {}
        section = None
        for line in wlf.readlines():
            line = line.strip()
            if len(line) > 0 and not line.startswith("#"):
                if line.startswith("["):
                    section = line[1:len(line) - 1]
                    if section not in whitelist:
                        whitelist[section] = []
                else:
                    whitelist[section].append(line)
        wlf.close()
        return whitelist

    def _createDB(self):
        conn = sqlite3.connect(self.db)
        connection = conn.cursor()
        connection.execute("create table md5 (md5 text)")
        connection.execute(IPv4WL.getCreateTable())
        connection.execute(IPv6WL.getCreateTable())
        connection.execute(IPv4SubnetWL.getCreateTable())
        connection.execute(IPv6SubnetWL.getCreateTable())
        connection.execute(DomainWL.getCreateTable())
        connection.execute(HostWL.getCreateTable())
        connection.execute(URLWL.getCreateTable())
        connection.close()
        conn.close()

    def isWhitelisted(self, indicatorType, indicator):
        """Return whether or not the specified indicator/indicatorType is whitelisted"""
        rv = False
        for wltype in self._indicatorMapping[indicatorType]:
            if not rv:
                wl = self._whitelists[wltype]
                if wl is None:
                    # Either throw an error or log a message
                    self._logger.error("Whitelist for type: {0} does not exist".format(wltype))
                    return False
                try:
                    rv = wl.isWhitelisted(self.conn, indicatorType, indicator)
                except Exception as e:
                    self._logger.error("Exception occurred while performing whitelist check")
                    self._logger.error(str(e))
        return rv



import re
import logging
from lqmt.lqm.data import AlertAction
from lqmt.lqm.unprocessed import UnprocessedAlertHandler
from lqmt.lqm.exceptions import ConfigurationError


class PullTool:
    def __init__(self, config):

        self._config = config  # the configuration object
        self.toolName = ""  # The name of the tool defined by the tools class

    def getConfig(self):
        return self._config

    def isEnabled(self):
        return self._config.isEnabled()

    def disable(self):
        """Disable the tool"""
        self._config.disable()

    def getName(self):
        nm = self._config.getName()
        if nm:
            return nm
        else:
            return "UNKNOWN-{0}".format(type(self).__name__)

    def getActionsToProcess(self):
        return None

    def initialize(self):  # TODO: Added to make work for pull tools
        pass


class PullToolConfig:
    """"Base class for all to tool configs"""

    def __init__(self, configData):
        self._logger = logging.getLogger("LQMT.PullToolConfig")
        self.configData = configData


class ToolConfig:
    """Base class for all tool configs"""

    def __init__(self, configData, csvToolInfo, unhandledCSV):
        self._logger = logging.getLogger("LQMT.ToolConfig")
        self._name = configData['name']  # the tool's name
        self._enabled = self._name is not None  # whether or not the tool is enabled
        self._unprocessedHandler = None  # The handler to use if there is an unprocessed alert
        self._hasError = False
        if 'unprocessed_file' in configData:
            if csvToolInfo is None or unhandledCSV is None:
                raise ConfigurationError("Missing csvToolInfo or unhandledCSV")
            cfg = {'file': configData['unprocessed_file']}
            cfg.update(unhandledCSV)
            self._unprocessedHandler = UnprocessedAlertHandler(csvToolInfo.create(cfg, None, None))

        self.configData = configData

    def getName(self):
        """
        :return: Returns tool named defined in the userconfig
        """
        return self._name

    def getError(self):
        return self._hasError

    def isEnabled(self):
        return self._enabled

    def disable(self):
        self._enabled = False

    def getActionsToProcess(self):
        return self._actionsToProcess

    def getUnprocessedHandler(self):
        return self._unprocessedHandler

    def validation(self, value, expected_type, required=False, default=None):
        """
        Function to validate values from the user configuration file.
        :param value: The value to extract from the configuration. This is a string used as a key in the configData dict
        :param expected_type: The expected type of the provided value
        :param required: True or False value indicating if this is a required value for successful configuration of the
         tool. If required is True and the value is not provided, an Error is thrown and the application halts.
        :param default: Value to be returned if the value is absent in the user configuration
        :return: Returns the validated value from the user configuration value. If a default is provided, that is
        returned in the event of the value being absent in the user config.
        """

        # if value not in config
        if value not in self.configData:
            if required:
                raise ConfigurationError(
                    "Missing the required parameter '{0}', in the Splunk tool: '{1}'".format(value, self.getName())
                )

            # if value is not in config, but there is a default, then return the default. Otherwise raise config error
            elif default is not None:
                self._logger.info(
                    "The '{0}' parameter wasn't specified in your config. Defaulting to value of '{1}'.".format(
                        value, default
                    )
                )
                return default

        # if value is in config
        if value in self.configData:
            # if value is of expected type return it, otherwise raise config error
            if isinstance(self.configData[value], expected_type):
                return self.configData[value]
            else:
                raise ConfigurationError(
                    "The '{0}' parameter was expected to be of the type {1}, but ended up being '{2}'.".format(
                        value, expected_type, type(self.configData[value])
                    )
                )
        else:
            raise ConfigurationError(
                "The '{0}' parameter is empty. Please provide a value for it.".format(value)
            )


class Tool:
    """The base class for all tools."""

    def __init__(self, config, alertActions):

        self._config = config  # the configuration object
        self._alertActions = set(alertActions)  # The alert actions this tool handles
        self.toolName = ""  # The name of the tool defined by the tools class
        self.dataFormat = []

    def getConfig(self):
        return self._config

    def isEnabled(self):
        return self._config.isEnabled()

    def disable(self):
        """Disable the tool"""
        self._config.disable()

    def getName(self):
        nm = self._config.getName()
        if nm:
            return nm
        else:
            return "UNKNOWN-{0}".format(type(self).__name__)

    def unprocessed(self, alert):
        """
        The specified alert has not been processed when it should have been.
        This gives the opportunity to log this info or report it to the user.
        """
        uph = self._config.getUnprocessedHandler()
        if uph is not None:
            uph.unprocessed(alert)

    def getActionsToProcess(self):
        return self._alertActions

    def initialize(self):
        uph = self._config.getUnprocessedHandler()
        if uph is not None:
            uph.initialize()

    def fileBegin(self):
        NotImplementedError

    def fileDone(self):
        NotImplementedError

    # Will only be called for alerts this tool can process
    def process(self, alert, meta=None):
        """Process the alert."""
        NotImplementedError

    def commit(self):
        """Called at the end of processing to allow the tool to perform any finalization"""
        NotImplementedError

    def cleanup(self):
        """called after commit to perform any cleanup necessary."""
        NotImplementedError

    @staticmethod
    def is_valid_ipv6(ip):
        """Validates IPv6 addresses."""
        pattern = re.compile(r"""
            ^
            \s*                         # Leading whitespace
            (?!.*::.*::)                # Only a single whildcard allowed
            (?:(?!:)|:(?=:))            # Colon iff it would be part of a wildcard
            (?:                         # Repeat 6 times:
                [0-9a-f]{0,4}           #   A group of at most four hexadecimal digits
                (?:(?<=::)|(?<!::):)    #   Colon unless preceeded by wildcard
            ){6}                        #
            (?:                         # Either
                [0-9a-f]{0,4}           #   Another group
                (?:(?<=::)|(?<!::):)    #   Colon unless preceeded by wildcard
                [0-9a-f]{0,4}           #   Last group
                (?: (?<=::)             #   Colon iff preceeded by exacly one colon
                 |  (?<!:)              #
                 |  (?<=:) (?<!::) :    #
                 )                      # OR
             |                          #   A v4 address with NO leading zeros
                (?:25[0-4]|2[0-4]\d|1\d\d|[1-9]?\d)
                (?: \.
                    (?:25[0-4]|2[0-4]\d|1\d\d|[1-9]?\d)
                ){3}
            )
            \s*                         # Trailing whitespace
            $
        """, re.VERBOSE | re.IGNORECASE | re.DOTALL)
        return pattern.match(ip) is not None

    @staticmethod
    def is_valid_ipv4(ip):
        """Validates IPv4 addresses.
        """
        pattern = re.compile(r"""
            ^
            (?:
              # Dotted variants:
              (?:
                # Decimal 1-255 (no leading 0's)
                [3-9]\d?|2(?:5[0-5]|[0-4]?\d)?|1\d{0,2}
              |
                0x0*[0-9a-f]{1,2}  # Hexadecimal 0x0 - 0xFF (possible leading 0's)
              |
                0+[1-3]?[0-7]{0,2} # Octal 0 - 0377 (possible leading 0's)
              )
              (?:                  # Repeat 0-3 times, separated by a dot
                \.
                (?:
                  [3-9]\d?|2(?:5[0-5]|[0-4]?\d)?|1\d{0,2}
                |
                  0x0*[0-9a-f]{1,2}
                |
                  0+[1-3]?[0-7]{0,2}
                )
              ){0,3}
            |
              0x0*[0-9a-f]{1,8}    # Hexadecimal notation, 0x0 - 0xffffffff
            |
              0+[0-3]?[0-7]{0,10}  # Octal notation, 0 - 037777777777
            |
              # Decimal notation, 1-4294967295:
              429496729[0-5]|42949672[0-8]\d|4294967[01]\d\d|429496[0-6]\d{3}|
              42949[0-5]\d{4}|4294[0-8]\d{5}|429[0-3]\d{6}|42[0-8]\d{7}|
              4[01]\d{8}|[1-3]\d{0,9}|[4-9]\d{0,8}
            )
            $
        """, re.VERBOSE | re.IGNORECASE)
        return pattern.match(ip) is not None


class ToolChain:
    """A ToolChain is a list of tools that pass data from one to the next"""

    def __init__(self, tools, name, enabled):
        self._tools = tools
        self._name = name
        self._enabled = True
        self._logger = logging.getLogger("LQMT.ToolChain")
        self._actionsToProcess = None
        for tool in self._tools:
            if self._actionsToProcess is None:
                self._actionsToProcess = tool.getActionsToProcess()
            else:
                # self._actionsToProcess &= tool.getActionsToProcess()
                # TODO: Needs some testing, believe this is a bug that prevents multiple tools with different Actions
                self._actionsToProcess |= tool.getActionsToProcess()
        self._alertsProcessed = 0
        self._alertsNotProcessed = 0

    def isEnabled(self):
        if self._enabled and self._tools:
            return self._enabled

    def initialize(self):
        """called by the controller at the beginning of processing to allow the tool chain to initialize itself."""
        # tell all the tools to initialize
        for tool in self._tools:
            tool.initialize()

    def fileBegin(self):
        """A new file is about to be processed."""
        if self.isEnabled():
            for tool in self._tools:
                tool.fileBegin()

    def fileDone(self):
        """All alerts from the current file have been processed."""
        if self.isEnabled():
            for tool in self._tools:
                tool.fileDone()

    def process(self, data, isWhitelisted, datafile, meta):
        """
        Process the alert data using each tool in the toolchain

        :param meta: used to provide parser with information not found in the source data.
        :param data: the processed alert data in the intermediate format
        :param isWhitelisted: indicates if the alert data is whitelisted or not
        :param datafile: the directory location of the processed alert datafile. Currently only used with FlexText
         because the alert has to be reprocessed for FlexText
        """
        if self.isEnabled():
            # if the alert can be processed by this toolchain, then process it
            if data.getAction() in self._actionsToProcess or AlertAction.get('All') in self._actionsToProcess:
                # if indicator isn't whitelisted, proceed with processing. Otherwise ignore processing
                # and log the whitelist block.
                if isWhitelisted is False:
                    # self._alertsProcessed += 1
                    alert_used = False
                    for tool in self._tools:
                        if type(data).__name__ in tool.dataFormat:
                            alert_used = True
                            # FlexText requires the datafile instead of the processed data.
                            if tool.toolName == "FlexText" or tool.toolName == 'MBL':
                                tool.process(datafile, meta)
                            else:
                                tool.process(data)

                    # if no tool maps to the format type, increment alert not processed
                    if alert_used is True:
                        self._alertsProcessed += 1
                    else:
                        self._alertsNotProcessed += 1
                else:
                    self._logger.info(
                        "Alert not processed. IP Indicator is whitelisted. Whitelisted IP:{0}".format(
                            data.getIPToBlock()))
            else:
                self._alertsNotProcessed += 1

    def pull_process(self):
        if self.isEnabled():
            for tool in self._tools:
                tool.initialize()
                tool.process(None)
                tool.commit()
                tool.cleanup()

    def commit(self):
        """Called at the end of processing to allow the tool chain to perform any finalization"""
        if self.isEnabled():
            for tool in self._tools:
                tool.commit()

    def cleanup(self):
        """Called after commit to perform any cleanup"""
        if self.isEnabled():
            self._logger.info(
                "Alerts processed:={0} AlertsNotProcessed={1}".format(self._alertsProcessed, self._alertsNotProcessed))
            for tool in self._tools:
                tool.cleanup()

    def updateEnabled(self):
        """
        Update the enabled state of this tool chain by checking all of its tools.
        If any are disabled, disable the chain, too.
        """
        en = self.isEnabled()
        if not en:
            return
        for tool in self._tools:
            if not tool.isEnabled():
                en = False
        self._enabled = en

    def getName(self):
        return self._name

    def printTools(self):
        toolStr = ""
        comma = " "
        for tool in self._tools:
            toolStr += comma
            toolStr += tool.getName()
            comma = ", "
        self._logger.debug("Tools in toolchain '{0}': {1}".format(self._name, toolStr))


from datetime import datetime
import re
from subprocess import Popen, PIPE, STDOUT
from lqmt.lqm.tool import ToolConfig
import sys
import logging
from lqmt.lqm.config import ConfigurationError


class CheckpointConfig(ToolConfig):
    def __init__(self, configData, csvToolInfo, unhandledCSV):
        super().__init__(configData, csvToolInfo, unhandledCSV)
        self._logger = logging.getLogger("LQMT.Checkpoint.{0}".format(self.getName()))
        hasError = False

        if ('hostname' in configData):
            self._hostname = configData['hostname']
        else:
            self._logger.error("hostname must be specified in the configuration")
            hasError = True
        if ('port' in configData):
            self._port = configData['port']
        else:
            self._logger.error("port must be specified in the configuration")
            hasError = True
        if ('username' in configData):
            self._username = configData['username']
        else:
            self._logger.error("username must be specified in the configuration")
            hasError = True
        if ('originator' in configData):
            self._originator = configData['originator']
        else:
            self._logger.error("originator must be specified in the configuration")
            hasError = True
        if ('default_duration' in configData):
            self._defaultDuration = configData['default_duration']
        else:
            self._defaultDuration = 3 * 24 * 3600
            self._logger.warning(
                "default_duration not specified in the configuration, setting default_duration to 86400 seconds")

        if (hasError):
            self.disable()
            raise ConfigurationError("Missing a required value in the user configuration for the to_checkpoint tool")

    def _openCPCommand(self, cmd, inp=None):
        args = ["ssh", self._username + "@" + self._hostname]
        for arg in cmd:
            args.append(arg)
        p = Popen(args, stdout=PIPE, stderr=STDOUT, stdin=PIPE)
        return p

    def _copy_file(self, file):
        args = ["scp", file, self._username + "@" + self._hostname + ":" + file, ]
        p = Popen(args, stdout=PIPE, stderr=STDOUT, stdin=PIPE)
        eof = False
        while (p.returncode == None or not eof):
            line = p.stdout.readline()
            eof = not line
            if (not eof):
                line = line.decode(sys.stdout.encoding).strip()
                if (line != "This system is for authorized use only."):
                    self._logger.info(line)
            p.poll()

    def updateRules(self, blockIPs, unblockUIDs):
        eventID = datetime.now().strftime("%Y%m%d-%H%M%S")

        # write rules to file
        f = open('cp.txt', 'w')
        for ip in blockIPs:
            f.write("add -t " + str(
                ip._timeout) + " -a r -l r -o " + self._originator + " -c " + eventID + " ip -d " + ip._addr + "\n")
            f.write("add -t " + str(
                ip._timeout) + " -a d -l r -o " + self._originator + " -c " + eventID + " ip -s " + ip._addr + "\n")

        for uid in unblockUIDs:
            f.write("del " + uid + "\n")

        f.close()

        # copy file to checkpoint
        self._copy_file("cp.txt")
        # run command on checkpoint
        p = self._openCPCommand(["fw samp batch < cp.txt"])
        eof = False
        while (p.returncode == None or not eof):
            line = p.stdout.readline()
            eof = not line
            if (not eof):
                line = line.decode(sys.stdout.encoding).strip()
                if (line != "This system is for authorized use only."):
                    self._logger.info(line)
            p.poll()

    def getRules(self):
        self._logger.info("Retrieving current rules for originator '{0}'".format(self._originator))
        regexp = re.compile("originator=" + self._originator)
        p = self._openCPCommand(["fw samp get"])
        eof = False
        cout = p.stdout
        self._rules = {}
        lines = []
        nr = 0
        while (p.returncode == None or not eof):
            line = cout.readline()
            line = line.decode(sys.stdout.encoding)
            eof = not line
            if (not eof):
                match = regexp.search(line)
                if (match):
                    nr = nr + 1
                    rec = self._parseRule(line)
                    if ('dst_ip_addr' in rec):
                        ip = rec['dst_ip_addr']
                        if (not ip in self._rules):
                            self._rules[ip] = dict()
                        self._rules[ip]['out'] = rec
                    else:
                        ip = rec['src_ip_addr']
                        if (not ip in self._rules):
                            self._rules[ip] = dict()
                        self._rules[ip]['in'] = rec
                else:
                    lines.append(line)
            p.poll()
        if (p.returncode == 255):  # ssh failed
            self._logger.error("Error with ssh - disbaling")
            for line in lines:
                self._logger.error(line.strip())
            self.disable()
        else:
            self._logger.info("Retrieved {0} rules".format(nr))
        return self._rules

    def _parseRule(self, line):
        d = {}
        for x in line.strip().split(" "):
            k, v = x.split("=")
            d[k] = v
        return d

    def getDefaultDuration(self):
        return self._defaultDuration




class Whitelist(object):
    """
    Abstract superclass for all whitelist types
    """
    
    def isWhitelisted(self,conn,indicatorType,indicator):
        """Return whether or not the indicator of type indicatorType is whitelisted by this whitelist"""
        raise NotImplementedError


"""
Created on March 13, 2018

@author: grjohnson
"""
import os
import logging
import toml
import ramrod
import jsonpath_rw
from stix.core import STIXPackage
from stix.common import information_source
from lqmt.lqm.data import StixFile


class STIXParser(object):
    def __init__(self, config=None):
        """
        Class for parsing STIX files.  Allows a user to do the following:
        - Filter files based on the information source
        - Pass the full raw file to a tool
        - Parse requested STIX elements into a dictionary object for a tool.
        - Parse COA rules (snort and yara) into full context and specific rule dictionaries for a tool.

        :param config: configuration file
        """
        # currently no config parameters
        self._logger = logging.getLogger("LQMT.Parsers")

        # accepted configuration values
        self._accepted_elements = ['stix_header', 'observables', 'indicators', 'ttps', 'exploit_targets', 'incidents',
                                   'courses_of_action', 'campaigns', 'threat_actors', 'related_packages']
        self._accepted_rules = ['snort', 'yara']

        # default some lists
        self._sources = []
        self._elements = []
        self._rules = []

        if config is not None:
            # parse config files
            for p, c in config.items():
                self.__parse_config(c)

    def __parse_config(self, configfile):
        """
        Extracts the defined configuration parameters for the parser.
        Important pre-condition is the user must configure elements and rules against a predefined enumeration.

        :param configfile: configuration file
        :return None:
        """
        if os.path.exists(configfile):
            cfg = open(configfile)
            topconfig = toml.loads(cfg.read())
            cfg.close()

            if 'Filters' in topconfig:
                if 'sources' in topconfig['Filters'][0]:
                    self._sources = topconfig['Filters'][0]['sources']

                if 'elements' in topconfig['Filters'][0]:
                    temp = topconfig['Filters'][0]['elements']
                    for x in temp:
                        if x.lower() in self._accepted_elements:
                            self._elements.append(x.lower())
                        else:
                            self._logger.warning("Incompatible STIX element configured " + x)

                if 'rules' in topconfig['Filters'][0]:
                    temp = topconfig['Filters'][0]['rules']
                    for x in temp:
                        if x.lower() in self._accepted_rules:
                            self._rules.append(x.lower())
                        else:
                            self._logger.warning("Incompatible rule for parser " + x)

    def __check_meta(self, meta):
        """
        Parses the Meta Data file looking for originators and custodians that match the user requested information source.

        :param meta: Meta data envelope for the file to be parsed
        :return Boolean: True if match is found, otherwise False.
        """
        ret = False

        # If there is no entry returns false for finding a match
        if 'DownloadElementExtendedAttribute' in meta:
            l = meta['DownloadElementExtendedAttribute']
            # this element can be either a list of dictionaries or a dictionary
            if type(l) is dict:
                if l['Field'] == 'Originator' or l['Field'] == 'Custodian':
                    if l['Value'] in self._sources:
                        return True
            elif type(l) is list:
                for x in l:
                    if x['Field'] == 'Originator' or x['Field'] == 'Custodian':
                        if x['Value'] in self._sources:
                            return True
            else:
                ret = False

        if ret is False:
            self._logger.info("Did not identify the requested source in meta-data file.")

        return ret

    def __check_info_source(self, datafile):
        """
        Parses the STIX Header element looking for an Information source element matching the configured value.
        Currently only matches to STIX Header, long-term could check every Information Source element.

        :param datafile: The file sent to be parsed, expected to be STIX XML format.
        :return Boolean: True if match is found, otherwise False.
        """
        ret = False

        try:
            # TODO: Parse only header or look for all Information_Source elements?
            # attempt to map as a STIX Package, if fails force an update with ramrod
            try:
                stix_package = STIXPackage.from_xml(datafile)
            except Exception as e:
                self._logger.warning("Unable to parse XML to STIX, attempting version change Exception={0}".format(e))
                updated = ramrod.update(datafile, force=True)
                stix_package = STIXPackage.from_xml(updated)

            # Compare the information source from the stix_header
            info_src = stix_package.stix_header.information_source
            if info_src.identity:
                if info_src.identity.name in self._sources:
                    return True

            # If there are contributing sources, parse and compare to source list
            if info_src.contributing_sources:
                if info_src.contributing_sources._inner_name == 'sources':
                    for x in info_src.contributing_sources._inner:
                        if type(x) is information_source.InformationSource:
                            if x.identity.name in self._sources:
                                return True
        except Exception as e:
            self._logger.warning(
                "Unable to force XML to STIX with ramrod. Unable to check source Exception={0}".format(e))
            return ret

        if ret is False:
            self._logger.info("Did not identify the requested source in STIX Header.")

        return ret

    def custom_parser(self, datafile):
        """
        Custom parser for the XML STIX file.
        Successful parsing results in a STIX File object for a tool containing the raw file, requested elements,
        and rule signatures found.

        :param datafile: The file sent to be parsed, expected to be STIX XML format.
        :return alert: A Stix File object containing the parsed elements for tools
        """
        ret = []
        rule_list = []
        el_list = []

        try:
            alert = StixFile()

            # open to set the raw file text in the alert object
            if os.path.exists(datafile):
                with open(datafile, 'r') as file:
                    data = file.read()
                    alert.setRawFile(data)

            try:
                # attempt to map as a STIX Package, if fails force an update with ramrod
                stix_package = STIXPackage.from_xml(datafile)
            except Exception as e:
                self._logger.warning("Unable to parse XML to STIX, attempting version change Exception={0}".format(e))
                updated = ramrod.update(datafile, force=True)
                stix_package = STIXPackage.from_xml(updated)

            stix_dict = stix_package.to_dict()

            # parse as a STIX package to find matching elements
            if self._elements:
                for i in self._elements:
                    test = jsonpath_rw.parse('$.' + str(i).lower())  # returns all JSON key matches
                    for match in test.find(stix_dict):
                        el_list.append(match.value)
                alert.setStixElements(el_list)

            # create dictionary with a key for each entry in rules list
            rules_dict = {}
            for y in self._rules:
                rules_dict[y] = []

            # parse the STIX package to find elements containing Rules
            if self._rules:
                # search STIX for all test mechanisms
                test = jsonpath_rw.parse('$..test_mechanisms')  # STIX defined element that contains rules
                for match in test.find(stix_dict):
                    rule_list.append(match.value[0])
                    # compare the XSI type to see if it's in the rules list 'xsi:type' = 'yaraTM:YaraTestMechanismType'
                    for y in self._rules:
                        if y.lower() in str(match.value[0]['xsi:type']).lower():  # STIX way to determine rule type
                            # append to list of rules for dictionary entry for "rule[s]" -- 'rule[s]' -> 'value'
                            if 'rule' in match.value[0]:
                                rules_dict[y].append(match.value[0]['rule']['value'])
                            elif 'rules' in match.value[0]:
                                for z in match.value[0]['rules']:
                                    rules_dict[y].append(z['value'])  # TODO: remove hardcoding
                # TODO: STIX Snort Specification allows -- Event Filters / Rate Filters / Event Suppression
                alert.setRules(rules_dict)
                alert.setFullRules(rule_list)

            # append the final alert
            ret.append(alert)

            self._logger.info("Identified {0} matching elements and {1} rule objects.".format(len(el_list), len(rule_list)))
        except Exception as e:
            self._logger.error("Error occurred parsing STIX file. Exception={0}".format(e))
            return ret

        return ret

    def parse(self, datafile, meta=None):
        """
        Parses a STIX File to return the raw file, requested elements, and rule signatures found.

        :param datafile: The STIX XML file to be parsed.
        :param meta: The meta data envelope for the STIX XML file to be parsed
        :return alerts: A list of STIX File objects parsed from the STIX XML file.
        """
        alerts = []
        source_match = False
        parse_data = False

        # Create the object file to pass to the tool chains
        try:
            self._logger.warning("Beginning to parse " + datafile)

            # if filtering by STIX information source
            if self._sources:
                # check both in the XML and in the meta data
                if meta:
                    if self.__check_meta(meta):
                        source_match = True
                    else:
                        source_match = self.__check_info_source(datafile)
                else:
                    source_match = self.__check_info_source(datafile)
            else:  # if no filters set then pass
                source_match = True

            # check for filter matches
            if source_match:
                parse_data = True
            else:
                self._logger.warning("Did not find match the requested source as the datafile producer.")

            if parse_data:
                alerts = self.custom_parser(datafile)

        except Exception as e:
            self._logger.error("LQMT-STIX-Parser: Problem with parsed data. Exception={0}".format(e))

        return alerts



from .whitelist import Whitelist
from netaddr import IPNetwork,IPSet

class IPv6SubnetWL(Whitelist):
    """IPv6 Subnet whitelist.
    This class is not yet implemented
    """

    @staticmethod
    def getCreateTable():
        return "create table ipv6sn (cidr char(40), minip integer(60), maxip integer(60) )"

    @staticmethod
    def storeDB(conn,subnets):
        """Overwrite the subnet info in the table with the specified subnets (string) in the database."""
        c = conn.cursor()
        vals = []
        for sn in subnets:
            cidr = IPNetwork(sn);
            vals.append((str(cidr), cidr.first, cidr.last))
        c.execute("delete from ipv6sn")
        c.executemany("insert into ipv6sn(cidr,minip,maxip) values (?,?,?)", iter(vals))
        c.close()

    def isWhitelisted(self,conn,indicatorType,indicator):
        raise NotImplementedError



#!/usr/bin/env python3


def main():
    import sys

    # need to put this check early so any imports farther down don't trigger an exception
    req_version = (3, 2)
    cur_version = sys.version_info
    if cur_version <= req_version:
        print("Your python version {0}.{1}.{2}-{3} is too old.  LQMTools requires at least version 3.2".format(
            cur_version[0], cur_version[1], cur_version[2], cur_version[3]))
        sys.exit(1)

    import logging
    import argparse
    import time
    from lqmt.lqm.logging import LQMLogging
    from lqmt.lqm.controller import LQMToolController

    parser = argparse.ArgumentParser()
    parser.add_argument('user_config_file', help='The user configuration file')
    args = parser.parse_args()

    logger = logging.getLogger("LQMT")
    try:
        controller = LQMToolController(args.user_config_file)
    except Exception as e:
        if not LQMLogging.isDebug():
            logger.error("An error occurred during configuration:")
        else:
            logger.exception("An error occurred during configuration:")
        logger.error(str(e))
        sys.exit(1)
    try:
        controller.run()
    except Exception as e:
        if not LQMLogging.isDebug():
            logger.error("An error occurred during processing:")
        else:
            logger.exception("An error occurred during processing:")
        logger.error(str(e))
    logger.info("LQMTool done. Process time: " + str(time.process_time()))


if __name__ == '__main__':
    main()



import logging
from lqmt.lqm.tool import ToolConfig


class FromSnortConfig(ToolConfig):
    def __init__(self, configData, csvToolInfo, unhandledCSV):
        super().__init__(configData, csvToolInfo, unhandledCSV)
        self.logger = logging.getLogger("LQMT.ToolName.{0}".format(self.getName()))

        modes = ['full', 'match']

        self.alert_paths = self.validation('alert_paths', list, required=True, default=['/var/log/snort/alert'])
        self.packets_paths = self.validation('packet_paths', list, required=True, default=['/nsm/sensor_data'])
        self.mode = self.validation('mode', str, default='full')
        if self.mode not in modes:
            self.mode = 'full'
        self.data_age = self.validation('max_file_age', str, required=True, default='5 minutes')
        self.rule_list = self.validation('rules', list, required=True, default=['/etc/nsm/rules/downloaded.rules'])
        self.result_path = self.validation('result_path', str, required=True, default='lqmt/results')
        # TODO: For now do not enable returning these snort context information
        # self.config_paths = self.validation('config_paths', list, default=['/etc/nsm'])
        # self.rules_paths = self.validation('rule_paths', list, default=['/etc/nsm/rules'])
        # self.tblsht_paths = self.validation('troubleshooting_paths', list, default=['/var/log/nsm'])
        # self.logs_paths = self.validation('log_paths', list, default=['/var/log/nsm'])



import argparse
import logging
import time
import sys
from lqmt.lqm.controller import LQMToolController
from lqmt.lqm.logging import LQMLogging


class LQMT(object):
    """
    API for LQMT. Currently a basic implementation allowing the ability to add a user configuration and
    the ability to run LQMT against it.
    """

    def __init__(self, config=None):
        self.logger = logging.getLogger("LQMT")
        self.user_config = config

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    def run(self, config=None):
        """
        Runs LQMT against user configuration.
        :param config: can supply directory path to user config directly here. Config file must be in toml format.
        """
        if config:
            self.user_config = config

        try:
            controller = LQMToolController(self.user_config)

        except Exception as e:
            if not LQMLogging.isDebug():
                self.logger.error("An error occurred during configuration:")
            else:
                self.logger.exception("An error occurred during configuration:")
            self.logger.error(str(e))
            sys.exit(1)

        try:
            controller.run()

        except Exception as e:
            if not LQMLogging.isDebug():
                self.logger.error("An error occurred during processing:")
            else:
                self.logger.exception("An error occurred during processing:")
            self.logger.error(str(e))
        logging.getLogger("LQMT").info("Process complete. Process time: " + str(time.process_time()))

    def add_config(self, config):
        """
        Adds user configuration file to be used when running LQMT
        :param config: Directory path to configuration file. Configuration file must be in TOML format.
        """
        self.user_config = config


def main():
    cur_version = sys.version_info
    if cur_version <= (3, 2):
        print("Your python version {0}.{1}.{2}-{3} is too old.  LQMTools requires at least version 3.2".format(
            cur_version[0], cur_version[1], cur_version[2], cur_version[3]
        ))
        sys.exit(1)

    parser = argparse.ArgumentParser(description="Parse CTI data and make it actionable in endpoint defense tools.")
    parser.add_argument('user_config_file', help='The path to the user configuration file. ')
    args = parser.parse_args()

    with LQMT(args.user_config_file) as lqmt:
        lqmt.run()

if __name__ == '__main__':
    main()



import logging
import requests
import time
import xmltodict
from xml.etree import ElementTree


def create_message(alert):
    """
    Creates a key-value formatted message for Splunk that contains parsed data from FlexT. Defaulting to parsing
    empty values.

    :param alert: Parsed alert data from FlexT
    :return: Returns formatted string.
    """
    data = alert.getAllFields(dictionary=True, parseEmpty=True)
    message = "{0} LQMT: ".format(time.asctime())
    for key, value in data.items():
        message += "{0}={1} ".format(key, value)

    return message


def formatUrlParam(parameter, value, addition=False):
    """
    Function used to format a string of URL parameters. Example output: "user=bob&title=builder"
    :param parameter: URL parameter that the value will be assigned to
    :param value: Value that will be assigned to the provided URL parameter
    :param addition: True or false value to indicate if this parameter is being added as an addition to other values.
    :return: If a valid value is given, a formatted string is returned. Otherwise an empty string is returned.
    """
    if addition:
        addition = "&"
    else:
        addition = ""

    if value:
        return "{}{}={}".format(addition, parameter, value)
    else:
        return ""


class ApiHandler:
    """
    Class for handling API calls to Splunk's REST Api. This class might end up being redundant depending on a few
    things, but that will be fleshed out further as the tool is built.
    """

    def __init__(self, host=None, port=None, username=None, password=None, splunk_token="", cert_check=True,
                 source=None, sourcetype=None, index=None, timeout_duration=0):
        self._messages_processed = 0
        self._logger = logging.getLogger("LQMT.Splunk.ApiCaller")
        self.host = host
        self.port = port
        self.source = formatUrlParam("source", source)
        self.sourcetype = formatUrlParam("sourcetype", sourcetype, True)
        self.index = formatUrlParam("index", index, True)
        self.cert_check = cert_check
        self.url = self.host + ":" + str(self.port)
        self.username = username
        self.password = password
        self.requests = requests
        self.authenticated = False
        self.splunk_token = {'Authorization': splunk_token}
        self.service = {
            'auth': "/services/auth/login/",
            'stream': "/services/receivers/stream/",
            'search': "/services/search/jobs"}
        self.headers = {}
        self.job_id = ""
        self.response = ""
        self.query = None
        self.timeout_duration = timeout_duration
        self.dispatch_states = ['QUEUED', 'PARSING', 'RUNNING', 'PAUSED', 'FINALIZING', 'FAILED', 'DONE']

        # Call authentication function when class object is created.
        self.authenticate()

    def __enter__(self):
        self.authenticate()
        return self

    def __exit__(self):
        self._logger.debug("Total messages processed: {0}".format(self._messages_processed))
        self.requests.post(url=self.url, headers={'Connection': 'close'}, verify=self.cert_check)

    def authenticate(self):
        """
        Method for authenticating against Splunk's REST Api. Returns a session token that will be used for future
        connections. If authentication fails, then the lqmt closes out with an error.
        :return: String: splunk_token. Value containing the splunk token provided by Splunk
        """
        # Check to make sure we aren't already authenticated
        if not self.authenticated:
            # Format username and password, then send the request
            data = {'username': self.username, 'password': self.password}
            r = self.requests.post(self.url + self.service['auth'], data=data, verify=self.cert_check)

            # If authentication is successful, then pull out the token and set our auth status. If not, weep.
            if r.ok:
                data = ElementTree.fromstring(r.content)
                self.splunk_token['Authorization'] = "Splunk " + data[0].text
                self.authenticated = True
                self._logger.debug(
                    "Successfully authenticated with Splunk instance. Token received: {0}".format(data[0].text)
                )
                # Format headers. Currently using splunks streaming input. Could be opened up later to let the user
                # choose.
                self.headers.update(self.splunk_token)
                self.headers.update({"x-splunk-input-mode": "streaming"})
            else:
                r.raise_for_status()

            return self.splunk_token

    def send_message(self, message, source=None, sourcetype=None, index=None):
        """
        Method for sending messages to Splunk via REST api. If the authentication function hasn't been run yet, then
        it is called.
        :param source: Used to override previously set source value
        :param sourcetype: Used to override previously set sourcetype
        :param index:
        :param message: message to be sent to splunk
        """

        # Override's for source, sourcetype, and index
        if source is not None:
            self.source = formatUrlParam("source", source)

        if sourcetype is not None:
            self.sourcetype = formatUrlParam("sourcetype", sourcetype, True)

        if index is not None:
            self.index = formatUrlParam("index", index, True)

        # If not authenticated, then authenticate
        if not self.authenticated:
            self.authenticate()

        # Build url for api and send the api request
        url = self.url + self.service['stream'] + "?{0}{1}{2}".format(
            self.source,
            self.sourcetype,
            self.index
        )

        r = self.requests.post(url, data=message, headers=self.headers, verify=self.cert_check)

        # If parsed successfully, tally and move on. Otherwise raise status
        if r.ok:
            self._messages_processed += 1
        else:
            r.raise_for_status()

        # self._logger.debug("Message sent to Splunk. "
        #                    "\nURL Used: '{0}'; "
        #                    "\nStatus code returned: '{1}';".format(url, r.status_code))

    def send_post_request(self, message, url):

        self.response = self.requests.post(url, data=message, headers=self.headers, verify=self.cert_check)

        # If parsed successfully, tally and move on. Otherwise raise status
        if self.response.ok:
            root = ElementTree.fromstring(self.response.text)
            self.job_id = root[0].text
            self._messages_processed += 1

        else:
            self.response.raise_for_status()
            self.job_id = "N/A"

        # self._logger.debug("Message sent to Splunk. "
        #                    "\nURL Used: '{0}'"
        #                    "\nMessage Sent: {1}"
        #                    "\nStatus code returned: '{2}'"
        #                    "\nJob ID: {3}".format(url, message, self.response.status_code, self.job_id))

        return self.job_id

    def fetch_job(self, job_id=None):
        """
        Used to fetch the results from a search job
        :param job_id: the Splunk job ID provided after you submit a search request. Defaults to None because if using
        the api, you most likely already have a job id assigned to self.job_id. Override if needed.
        :return:
        """
        job_status = "PENDING"
        sleep_inc = 1
        if job_id is not None:
            self.job_id = job_id

        if self.response.ok:
            while job_status != "DONE":
                self._logger.debug("Fetching Job - Job still pending.")
                status_response = requests.get(self.url + "/services/search/jobs/" + job_id + "/",
                                               auth=(self.username, self.password),
                                               verify=self.cert_check)

                if status_response.status_code == 200 and status_response.ok:
                    parsed_state = self.parse_xml_for_state(status_response.text)
                    if parsed_state in self.dispatch_states:
                        job_status = parsed_state

                if job_status != "DONE":
                    # TODO: Update sleep function
                    time.sleep(sleep_inc)
                    sleep_inc += 2

                elif sleep_inc > self.timeout_duration:
                    pass
                    self._logger.exception("Splunk search job has exceeded your defined timeout duration of {0}".format(
                        self.timeout_duration
                    ))
        else:
            self._logger.error("Error authenticating request.")

        self._logger.debug("Job Finished. Fetching results.")
        payload = {'output_mode': 'csv'}
        job_result = requests.get(self.url + "/services/search/jobs/" + job_id + "/results/",
                                  auth=(self.username, self.password), verify=False, params=payload)

        return job_result

    def getTotalMessagesProcessed(self):
        return self._messages_processed

    def build_url(self, host, port, service, params):
        url = host + ":" + str(port) + self.service[service] + params

        return url

    @staticmethod
    def write_file(response, file):
        if response:
            f = open(file, 'w')
            f.write(response.text)
            f.close()

    @staticmethod
    def format_url_params(params):
        url_params = "?"
        params = list(filter(None, params))
        for param in params:
            url_params += param + "&"

        return url_params[:-1]

    @staticmethod
    def parse_xml_for_state(content):
        value = None
        xml_dict = xmltodict.parse(content)
        for item in xml_dict['entry']['content']['s:dict']['s:key']:
            if '@name' in item:
                if item['@name'] == "dispatchState":
                    value = item['#text']
        return value


"""
Created on Jan 12, 2015

@author: taxon
"""
import FlexTransform
from lqmt.lqm.data import Alert
import os
import inspect
import logging
import io


class FlexTransformParser(object):
    def __init__(self, config=None):
        self._logger = logging.getLogger("LQMT.Parsers")

        # get the FlexTransform directory
        self._current_dir, name = os.path.split(inspect.getfile(FlexTransform.FlexTransform))
        self._transform = FlexTransform.FlexTransform.FlexTransform()
        self._exceptions = (
            'stix-tlp',
            'STIX',
            'IIDactiveBadHosts',
            'IIDdynamicBadHosts',
            'IIDrecentBadIP',
            'IIDcombinedURL'
        )

        # config
        if config is not None:
            for p, c in config.items():
                self.add_parser(p, c)

    def add_parser(self, parserName, parserConfiguration):
        """
        Function that extends the AddParser function from FlexTransform.
        :param parserName: Name of the parser being added
        :param parserConfiguration: Path to the configuration file being added
        """
        incompat_list = ['STIXparserConfig', 'RuleParserConfig']
        if parserName not in self._transform.Parsers:
            if parserName not in incompat_list:  # TODO: added to skip different parser config
                config_file = open(os.path.join(self._current_dir, parserConfiguration), 'r')
                self._transform.add_parser(parserName, config_file)

    def parse(self, datafile, meta=None):
        """
        Parse the datafile using the metadata dictionary.

        Note: Removed sourceMetaData from the parser for now due to conflicts with STIX-TLP. Currently assessing
        if it needs to be included. Currently the meta file is just being used to identify the sourceParser.

        :param datafile: Contains path to the file containing the alert data.
        :param meta: Contains meta data about the datafile. Examples includes PayloadFormat, FileName, PayloadType, and
        more.
        :return: Returns parsed alert data.
        """
        alerts = []

        # TODO: Stix-tlp parser currently doesn't support meta files. Until it does, meta files are for the cfm format

        try:
            data = self.custom_parser(datafile, meta['PayloadFormat'], 'LQMTools', meta=meta)

        except Exception as e:
            data = []
            self._logger.error(
                "LQMT-FlexTransform-Parser: Error parsing file file='{0}' exception='{1}'".format(datafile, e))
        try:
            for d in data:
                alert = Alert()
                alert.setFromDict(d)
                alerts.append(alert)
        except Exception as e:
            self._logger.error("LQMT-FlexTransform-Parser: Problem with parsed data. Exception={0}".format(e))
        return alerts

    def parseflextext(self, datafile, meta, destination_file_obj, config_str):
        """
        Flextext parser. Transforms intermediate data into a user defined structure.

        :param datafile: dir path to the alert datafile that will be transformed
        :param meta: Contains meta data about the datafile. Examples includes PayloadFormat, FileName, PayloadType, and
        more.
        :param destination_file_obj: File object used to write contents to file.
        :param config_str: Parser configuration in the form of a string. Is wrapped in an IO wrapped and then passed
        to FlexT to configure FlexText parser
        :return: Returns none. Data is outputted to file specified by destination_file_obj
        """

        # Add FlexText parser using config_str wrapped in StringIO object
        if 'FlexText' not in self._transform.Parsers:
            config_str_io = io.StringIO(config_str)
            config_str_txt = io.StringIO(config_str_io.read())
            self._transform.add_parser("FlexText", config_str_txt)

        # Run FlexText parser
        try:
            self.custom_parser(datafile, meta['PayloadFormat'], 'FlexText', target_file=destination_file_obj, meta=meta)

        except Exception as e:
            self._logger.error(
                "LQMT-FlexText-Parser: Error parsing file file='{0}' exception='{1}'".format(datafile, e))

    def custom_parser(self, datafile, payloadformat, targetparser, target_file=None, meta=None):
        """
        Custom parser wrapper for the flex_transform transform function.

        :return: returns the parsed data when a target_file is not specified
        """
        output = None

        # check against the list of file types that currently have exceptions for meta data files
        if meta['PayloadFormat'] in self._exceptions:
            meta = None

        try:
            # if datafile is already an io object, use it
            if isinstance(datafile, io.StringIO):
                output = self._transform.transform(
                    datafile,
                    payloadformat,
                    targetparser,
                    target_file=target_file,
                    source_meta_data=meta
                )
            # else, check if it is a valid file path. If so, open it for parsing.
            elif os.path.exists(datafile):
                with open(datafile, 'r') as file:
                    output = self._transform.transform(
                        file,
                        payloadformat,
                        targetparser,
                        target_file=target_file,
                        source_meta_data=meta
                    )
        except Exception as e:
            self._logger.error("LQMT-Custom-Parser: Error parsing file. file='{0}' exception='{1}'".format(datafile, e))

        return output



import logging
from lqmt.lqm.tool import ToolConfig


class MBLConfig(ToolConfig):
    def __init__(self, configData, csvToolInfo, unhandledCSV):
        super().__init__(configData, csvToolInfo, unhandledCSV)
        self.logger = logging.getLogger("LQMT.ToolName.{0}".format(self.getName()))

        self.source_configs = {
            'Cfm13Alert': 'resources/sampleConfigurations/cfm13.cfg',
            'Cfm20Alert': 'resources/sampleConfigurations/cfm20alert.cfg',
            'stix-tlp': 'resources/sampleConfigurations/stix_tlp.cfg'
        }

        self.host = self.validation('host', str, required=True)
        self.port = self.validation('port', int, default=8089)
        self.username = self.validation('username', str, required=True)
        self.password = self.validation('password', str, required=True)
        self.cert_check = self.validation('cert_check', bool, default=True)
        self.source = self.validation('source', str, default="lqmt")
        self.sourcetype = self.validation('sourcetype', str, default="")
        self.index = self.validation('index', str)
        self.override_parser = self.validation('override_parser', str, default="MBL")



from lqmt.lqm.tool import Tool
import logging
from lqmt.lqm.data import AlertAction


class Block():
    def __init__(self, ip, timeout):
        self._addr = ip
        self._timeout = timeout


class ToCheckpoint(Tool):
    originator = "CFM"

    def __init__(self, config):
        super().__init__(config, [AlertAction.get('Block'), AlertAction.get('Revoke')])
        self._logger = logging.getLogger("LQMT.Checkpoint.{0}".format(self.getName()))
        self._rules = {}  # CFM-created rules from config device
        self._blocks = set()  # Blocks to block that aren't already blocked
        self._blockIPs = set()
        self._unblockUIDs = set()  # IPs to unblock
        self._totalProcessed = 0
        self._totalBlocked = 0
        self._totalRevoked = 0
        self._modified = False
        self._cp = config

    def initialize(self):
        super().initialize()
        self._rules = self._cp.getRules()

    def process(self, alert):
        self._totalProcessed += 1
        action = alert.getAction()
        if action == AlertAction.get('Block'):
            self._block(alert)
        elif action == AlertAction.get('Revoke'):
            self._revoke(alert)

    def commit(self):
        # Possibly look for the refresh process before writing file?
        self._logger.info("Updating checkpoint")
        self._cp.updateRules(self._blocks, self._unblockUIDs)

    def cleanup(self):
        self._logger.info(
                "Processed {0} alerts.  New blocks: {1}  New revoke: {2}".format(self._totalProcessed,
                                                                                 self._totalBlocked,
                                                                                 self._totalRevoked))

    def _block(self, alert):
        addr = alert.getIPToBlock()
        if addr is None:
            self.unprocessed(alert)
            return
        if self.is_valid_ipv4(addr) or self.is_valid_ipv6(addr):
            if addr not in self._rules and addr not in self._blockIPs:
                self._blocks.add(Block(addr, self._getDuration(alert)))
                self._blockIPs.add(addr)
                self._totalBlocked += 1
                self._modified = True
                return True
        return False

    def _getDuration(self, alert):
        duration = alert.getDuration1()
        if duration is None or int(duration) == 0:
            return self._config.getDefaultDuration()
        return duration

    def _revoke(self, alert):
        addr = alert.getIPToBlock()
        if addr is None:
            self.unprocessed(alert)
            return
        if self.is_valid_ipv4(addr) or self.is_valid_ipv6(addr):
            if addr in self._rules:
                self._unblockUIDs.add(self._rules[addr]['in']['uid'])
                self._unblockUIDs.add(self._rules[addr]['out']['uid'])
                self._totalRevoked += 1
                self._modified = True
                return True
        return False

    def unblockAll(self):
        self.loadDeviceConfig()
        for addr in self._rules:
            if 'in' not in self._rules[addr]:
                self._logger.info(("No inbound block for addr= %s" % addr))
            else:
                self._unblockUIDs.add(self._rules[addr]['in']['uid'])
            if 'out' not in self._rules[addr]:
                self._logger.info(("No outbound block for addr= %s" % addr))
            else:
                self._unblockUIDs.add(self._rules[addr]['out']['uid'])
        self.commit()



from .whitelist import Whitelist

class DomainWL(Whitelist):
    """Domain whitelist.
    This class stores its whitelisted domains in the domian table.  If a domain is in the table it is whitelisted.
    """

    @staticmethod
    def getCreateTable():
        """Create the table to hold the domains"""
        return "create table domain (domain char(256))"

    @staticmethod
    def storeDB(conn,domains):
        """Overwrite the domains in the table with the specified domains (string) in the database."""
        c=conn.cursor()
        vals=[]
        for domain in domains:
            vals.append( (domain,) )
        c.execute("delete from domain")
        c.executemany("insert into domain(domain) values (?)",iter(vals))
        c.close()

    def isWhitelisted(self,conn,indicatorType,indicator):
        """Return whether or not the indicator of type indicatorType is whitelisted by this whitelist.
        This will check for all subdomains as well."""
        c=conn.cursor()
        # sqlite: || is concatenation
        c.execute("select domain from domain where ? like '%' || domain",(("%"+indicator),))
        rec=c.fetchone()
        c.close()
        return rec != None



import toml
import json
import arrow
from unittest import TestCase, main
from lqmt.test.test_data.filters.filter_configs import TESTCONFIG1
from lqmt.test.test_data.filters.filter_inputs import INPUT1
from lqmt.lqm.sourcefilter import SourceFilters


class TestFilters(TestCase):
    def setUp(self):
        self.user_config = toml.loads(TESTCONFIG1)
        self.filters = SourceFilters(self.user_config['Source']['Filters'][0])

    def test_all_filters(self):
        meta = json.loads(INPUT1)
        # true case
        meta['SentTimestamp'] = str(arrow.utcnow().timestamp)
        self.assertTrue(self.filters.checkAllFilters(meta))
        # fail sending site
        meta['SendingSite'] = 'TEST3'
        self.assertFalse(self.filters.checkAllFilters(meta))
        # fail payload type
        meta = json.loads(INPUT1)
        meta['PayloadType'] = 'Report'
        self.assertFalse(self.filters.checkAllFilters(meta))
        # fail payload format
        meta = json.loads(INPUT1)
        meta['PayloadFormat'] = 'OtherPDF'
        self.assertFalse(self.filters.checkAllFilters(meta))
        # fail sensitivity
        meta = json.loads(INPUT1)
        meta['DataSensitivity'] = 'OUO'
        self.assertFalse(self.filters.checkAllFilters(meta))
        # fail restrictions
        meta = json.loads(INPUT1)
        meta['SharingRestrictions'] = 'AMBER'
        self.assertFalse(self.filters.checkAllFilters(meta))
        # fail recon policy
        meta = json.loads(INPUT1)
        meta['ReconPolicy'] = 'Touch'
        self.assertFalse(self.filters.checkAllFilters(meta))
        # fail file age
        meta = json.loads(INPUT1)
        meta['SentTimestamp'] = str(arrow.utcnow().shift(months=-2, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkAllFilters(meta))

    def test_site_includes(self):
        meta = json.loads(INPUT1)
        # test site includes
        self.assertTrue(self.filters.checkSendingSite(meta))
        # test lower case
        meta['SendingSite'] = 'TEST3'
        self.assertFalse(self.filters.checkSendingSite(meta))
        # test empty
        self.filters._site_includes = []
        self.assertTrue(self.filters.checkSendingSite(meta))

    def test_site_excludes(self):
        meta = json.loads(INPUT1)
        self.filters._site_includes = []
        # check includes empty and site not in excludes
        self.assertTrue(self.filters.checkSendingSite(meta))
        # check site in the excludes list
        meta['SendingSite'] = 'SITE2'
        self.assertFalse(self.filters.checkSendingSite(meta))
        # check empty excludes list
        self.filters._site_excludes = []
        self.assertTrue(self.filters.checkSendingSite(meta))

    def test_payload_types(self):
        meta = json.loads(INPUT1)
        # test type in the list
        self.assertTrue(self.filters.checkPayloadType(meta))
        # test unexpected list
        meta['PayloadType'] = 'Report'
        self.assertFalse(self.filters.checkPayloadType(meta))
        # test empty list
        self.filters._payload_types = []
        self.assertTrue(self.filters.checkPayloadType(meta))

    def test_payload_formats(self):
        meta = json.loads(INPUT1)
        # test format in the list
        self.assertTrue(self.filters.checkPayloadFormat(meta))
        # test unexpected list
        meta['PayloadFormat'] = 'OtherPDF'
        self.assertFalse(self.filters.checkPayloadFormat(meta))
        # test empty list
        self.filters._payload_formats = []
        self.assertTrue(self.filters.checkPayloadFormat(meta))

    def test_sensitivities(self):
        meta = json.loads(INPUT1)
        # test format in the list
        self.assertTrue(self.filters.checkDataSensitivity(meta))
        # test unexpected list
        meta['DataSensitivity'] = 'OUO'
        self.assertFalse(self.filters.checkDataSensitivity(meta))
        # test empty list
        self.filters._sensitivities = []
        self.assertTrue(self.filters.checkDataSensitivity(meta))

    def test_restrictions(self):
        meta = json.loads(INPUT1)
        # test format in the list
        self.assertTrue(self.filters.checkSharingRestrictions(meta))
        # test unexpected list
        meta['SharingRestrictions'] = 'AMBER'
        self.assertFalse(self.filters.checkSharingRestrictions(meta))
        # test empty list
        self.filters._restrictions = []
        self.assertTrue(self.filters.checkSharingRestrictions(meta))

    def test_reconn(self):
        meta = json.loads(INPUT1)
        # test format in the list
        self.assertTrue(self.filters.checkReconPolicy(meta))
        # test unexpected list
        meta['ReconPolicy'] = 'Touch'
        self.assertFalse(self.filters.checkReconPolicy(meta))
        # test empty list
        self.filters._reconnaissance = []
        self.assertTrue(self.filters.checkReconPolicy(meta))

    def test_no_file_age(self):
        meta = json.loads(INPUT1)
        # check no setting
        self.filters._max_file_age = None
        self.assertTrue(self.filters.checkFileAge(meta))

    def test_file_age(self):
        # utilizes tests for month
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(months=-2, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))

    def test_file_age_sec(self):
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        self.filters._max_file_age = "30 secs"
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(seconds=-35).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))

    def test_file_age_min(self):
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        self.filters._max_file_age = "30 min"
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(minutes=-30, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))

    def test_file_age_hour(self):
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        self.filters._max_file_age = "1 hour"
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(hours=-1, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))

    def test_file_age_day(self):
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        self.filters._max_file_age = "2 days"
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(days=-2, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))

    def test_file_age_week(self):
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        self.filters._max_file_age = "1 week"
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(weeks=-1, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))

    def test_file_age_yr(self):
        meta = json.loads(INPUT1)
        now = arrow.utcnow()
        self.filters._max_file_age = "3 yrs"
        meta['SentTimestamp'] = str(now.timestamp)
        # check true
        self.assertTrue(self.filters.checkFileAge(meta))
        # check false
        meta['SentTimestamp'] = str(now.shift(years=-3, seconds=-10).timestamp)
        self.assertFalse(self.filters.checkFileAge(meta))


if __name__ == '__main__':
    main()



import socket
from lqmt.lqm.tool import Tool
from datetime import datetime
import re
import logging
from lqmt.lqm.data import AlertAction


class Header():
    """CEF Header object - contains the fields needed to create the CEF header"""

    def __init__(self, alert):
        # necessary escapes
        self._replacer = multiple_replacer(('|', '\|'), ('\\', '\\\\'))
        self._version = 0
        self._deviceVendor = "DOECFM"
        self._deviceProduct = "ToCEF"
        self._deviceVersion = "1.0.1"
        self._signatureID = alert.getReason()
        self._name = alert.getReason()
        self._severity = 5
        self._timestamp = alert.getProcessedTime()

    def toCEFString(self):
        """Outout the header as a properly formatted CEF header."""
        ts = datetime.fromtimestamp(int(self._timestamp))
        s = ts.strftime("%b %d %H:%M:%S")
        s += " " + socket.gethostname()
        s += " CEF:{0}|{1}|{2}|{3}|{4}|{5}|{6}".format(self._version, self._escape(self._deviceVendor),
                                                       self._escape(self._deviceProduct),
                                                       self._escape(self._deviceVersion),
                                                       self._escape(self._signatureID), self._escape(self._name),
                                                       self._severity)
        return s

    def _escape(self, s):
        return self._replacer(s)


class Record:
    def _toDate(self, unixtime):
        if (unixtime == None):
            return None
        ts = datetime.fromtimestamp(int(unixtime))
        return ts.strftime("%b %d %Y %H:%M:%S")

    def __init__(self, alert):
        self._alert = alert
        self._replacer = multiple_replacer(('=', '\\='), ('\\', '\\\\'), ('\r\n', '\\n'), ('\n', '\\n'), ('\r', '\\n'))
        self._initFields()
        self._header = Header(alert)
        self._externalID = alert.getDataItemID()
        self._start = self._toDate(alert.getDetectedTime())
        self._rt = self._toDate(alert.getProcessedTime())
        self._src = alert.getSourceIP()
        self._dst = alert.getDestIP()
        self._request = alert.getURL()
        self._shost = alert.getSourceHost()
        self._dhost = alert.getDestHost()
        self._spt = alert.getSourcePort()
        self._dpt = alert.getDestPort()
        self._proto = alert.getTransportProtocol()
        self._deviceExternalID = alert.getDirectSource()
        self._act = alert.getAction()
        self._end = self._toDate(alert.getEndTime())
        self._cat = alert.getReason()
        self._priors = alert.getPriors()
        self._msg = alert.getComment()
        self._extensions = dict()
        self._extensions['Restrictions'] = alert.getRestriction()
        self._extensions['Sensitivity'] = alert.getSensitivity()
        self._extensions['ReconAllowed'] = alert.getReconAllowed()
        self._extensions['FileID'] = alert.getFileID()
        self._extensions['ReportedTime'] = self._toDate(alert.getReportedTime())
        self._extensions['SecondarySource'] = alert.getSecondarySource()
        self._extensions['Confidence'] = alert.getConfidence()
        self._extensions['Severity'] = alert.getSeverity()
        self._extensions['Relevance'] = alert.getRelevance()
        self._extensions['RelatedID'] = alert.getRelatedID()
        self._extensions['RelationType'] = alert.getRelationType()
        self._extensions['FileHasMore'] = alert.getFileHasMore()

    def _initFields(self):
        self._fields = []
        self._fields.append('externalID')
        self._fields.append('start')
        self._fields.append('rt')
        self._fields.append('src')
        self._fields.append('dst')
        self._fields.append('request')
        self._fields.append('shost')
        self._fields.append('dhost')
        self._fields.append('spt')
        self._fields.append('dpt')
        self._fields.append('proto')
        self._fields.append('deviceExternalID')
        self._fields.append('act')
        self._fields.append('end')
        self._fields.append('cat')
        self._fields.append('priors')
        self._fields.append('msg')

    def toCEFString(self):
        """Create the CEF string.  Only include fields that aren't None"""
        s = self._header.toCEFString()
        sep = "|"
        for ofld in self._fields:
            fld = "_" + ofld
            if hasattr(self, fld):
                val = getattr(self, fld)
                if val != None:
                    s += sep + self._escape(ofld) + "=" + self._escape(val)
                    sep = " "
        for fld, val in self._extensions.items():
            if val is not None:
                s += sep + self._escape(fld) + "=" + self._escape(val)
                sep = " "
        self._logger.debug("CEF String returned: '{0}'".format(s))
        return s

    def _escape(self, s):
        return self._replacer(s)


class ToCEF(Tool):
    def __init__(self, config):
        super().__init__(config, [AlertAction.get('All')])
        self._logger = logging.getLogger("LQMT.CEF.{0}".format(self.getName()))

    def initialize(self):
        super().initialize()

    def process(self, data):
        try:
            self._logger.debug("CEF processing started")
            return Record(data)
        except Exception as e:
            self._logger.error("Error occurred in creating CEF data")
            self._logger.error(str(e))

    def commit(self):
        pass

    def cleanup(self):
        pass


def multiple_replacer(*key_values):
    replace_dict = dict(key_values)
    replacement_function = lambda match: replace_dict[match.group(0)]
    pattern = re.compile("|".join([re.escape(k) for k, v in key_values]), re.M)
    return lambda string: pattern.sub(replacement_function, string)


def multiple_replace(string, *key_values):
    return multiple_replacer(*key_values)(string)



CFM13ALERT = """
<!DOCTYPE IDMEF-Message PUBLIC "-//IETF//DTD RFC XXXX IDMEF v1.0//EN" "idmef-message.dtd">
    <IDMEF-Message xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://www.anl.gov/cfm/1.3/IDMEF-Message" xsi:schemaLocation="http://www.anl.gov/cfm/1.3/IDMEF-Message/../../../resources/schemas/CFMMessage13.xsd">
      <Alert>
        <Analyzer analyzerid="Fake">
          <Node>
            <location>Fake National Lab</location>
            <name>Fake Name</name>
          </Node>
        </Analyzer>
        <AnalyzerTime>2016-02-21T22:50:02+0600</AnalyzerTime>
        <AdditionalData meaning="report schedule" type="string">5 minutes</AdditionalData>
        <AdditionalData meaning="number of alerts in this report" type="integer">2</AdditionalData>
        <AdditionalData meaning="report type" type="string">alerts</AdditionalData>
        <AdditionalData meaning="report start time" type="date-time">2016-02-21T22:45:53+0700</AdditionalData>
      </Alert>
      <Alert>
        <CreateTime>2016-02-21T22:45:53-0400</CreateTime>
        <Source>
          <Node>
            <Address category="ipv4-addr">
              <address>10.10.10.10</address>
            </Address>
          </Node>
        </Source>
        <Target>
          <Service>
            <port>22</port>
            <protocol>TCP</protocol>
          </Service>
        </Target>
        <Classification text="SSH scans against multiple hosts, direction:ingress, confidence:87, severity:high">
          <Reference meaning="Scanning" origin="user-specific">
            <name>SSH Attack</name>
            <url> </url>
          </Reference>
        </Classification>
        <Assessment>
          <Action category="block-installed"/>
        </Assessment>
        <AdditionalData meaning="restriction" type="string">private</AdditionalData>
        <AdditionalData meaning="recon" type="integer">0</AdditionalData>
        <AdditionalData meaning="prior offenses" type="integer">11</AdditionalData>
        <AdditionalData meaning="duration" type="integer">86400</AdditionalData>
        <AdditionalData meaning="alert threshold" type="integer">0</AdditionalData>
        <AdditionalData meaning="OUO" type="integer">0</AdditionalData>
        <AdditionalData meaning="top level domain owner" type="string">The Republic of Fake</AdditionalData>
      </Alert>
    </IDMEF-Message>
    """

CFM20ALERT = """
<CFMAlert xmlns="http://www.anl.gov/cfm/2.0/current/CFMAlert">
    <Version>2.0</Version>
    <Alert>
        <AlertID>1cdf6f8f-20da-488e-9132-bbf850621418</AlertID>
        <AlertTimestamp>1468350602</AlertTimestamp>
        <IndicatorSet>
            <Indicator>
                <Type>ipv6sourceaddress</Type>
                <Constraint>IPv6ColonHexEquality</Constraint>
                <Value>8675:a289:5:102c::bd8:baac</Value>
            </Indicator>
        </IndicatorSet>
        <ReasonList>
            <Reason>
                <ReasonCategory>Exploit</ReasonCategory>
                <ReasonDescription>The WAF detected a scan for vulnerable web applications</ReasonDescription>
            </Reason>
        </ReasonList>
        <ActionList>
            <Action>
                <ActionCategory>Block</ActionCategory>
                <ActionTimestamp>1468350602</ActionTimestamp>
            </Action>
        </ActionList>
        <AlertExtendedAttribute>
            <Field>confidence</Field>
            <Value>80</Value>
        </AlertExtendedAttribute>
        <AlertExtendedAttribute>
            <Field>severity</Field>
            <Value>high</Value>
        </AlertExtendedAttribute>
    </Alert>
</CFMAlert>
"""

STIX = """<stix:STIX_Package
        xmlns:cyboxCommon="http://cybox.mitre.org/common-2"
        xmlns:cybox="http://cybox.mitre.org/cybox-2"
        xmlns:cyboxVocabs="http://cybox.mitre.org/default_vocabularies-2"
        xmlns:AddressObj="http://cybox.mitre.org/objects#AddressObject-2"
        xmlns:ArtifactObj="http://cybox.mitre.org/objects#ArtifactObject-2"
        xmlns:FileObj="http://cybox.mitre.org/objects#FileObject-2"
        xmlns:URIObj="http://cybox.mitre.org/objects#URIObject-2"
        xmlns:marking="http://data-marking.mitre.org/Marking-1"
        xmlns:tlpMarking="http://data-marking.mitre.org/extensions/MarkingStructure#TLP-1"
        xmlns:indicator="http://stix.mitre.org/Indicator-2"
        xmlns:stixCommon="http://stix.mitre.org/common-1"
        xmlns:stixVocabs="http://stix.mitre.org/default_vocabularies-1"
        xmlns:stix="http://stix.mitre.org/stix-1"
        xmlns:CFM="http://www.anl.gov/cfm/stix"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="
        http://cybox.mitre.org/common-2 http://cybox.mitre.org/XMLSchema/common/2.1/cybox_common.xsd
        http://cybox.mitre.org/cybox-2 http://cybox.mitre.org/XMLSchema/core/2.1/cybox_core.xsd
        http://cybox.mitre.org/default_vocabularies-2 http://cybox.mitre.org/XMLSchema/default_vocabularies/2.1/cybox_default_vocabularies.xsd
        http://cybox.mitre.org/objects#AddressObject-2 http://cybox.mitre.org/XMLSchema/objects/Address/2.1/Address_Object.xsd
        http://cybox.mitre.org/objects#ArtifactObject-2 http://cybox.mitre.org/XMLSchema/objects/Artifact/2.1/Artifact_Object.xsd
        http://cybox.mitre.org/objects#FileObject-2 http://cybox.mitre.org/XMLSchema/objects/File/2.1/File_Object.xsd
        http://cybox.mitre.org/objects#URIObject-2 http://cybox.mitre.org/XMLSchema/objects/URI/2.1/URI_Object.xsd
        http://data-marking.mitre.org/Marking-1 http://stix.mitre.org/XMLSchema/data_marking/1.1.1/data_marking.xsd
        http://data-marking.mitre.org/extensions/MarkingStructure#TLP-1 http://stix.mitre.org/XMLSchema/extensions/marking/tlp/1.1.1/tlp_marking.xsd
        http://stix.mitre.org/Indicator-2 http://stix.mitre.org/XMLSchema/indicator/2.1.1/indicator.xsd
        http://stix.mitre.org/common-1 http://stix.mitre.org/XMLSchema/common/1.1.1/stix_common.xsd
        http://stix.mitre.org/default_vocabularies-1 http://stix.mitre.org/XMLSchema/default_vocabularies/1.1.1/stix_default_vocabularies.xsd
        http://stix.mitre.org/stix-1 http://stix.mitre.org/XMLSchema/core/1.1.1/stix_core.xsd" id="CFM:STIXPackage-21856f56-eb97-50ca-bfb0-bd425e3d01c0" version="1.1.1" timestamp="2016-03-29T19:33:13+03:00">
        <stix:STIX_Header>
            <stix:Title>Test PDF</stix:Title>
            <stix:Package_Intent xsi:type="stixVocabs:PackageIntentVocab-1.0">Indicators</stix:Package_Intent>
            <stix:Description>Ransomware Update</stix:Description>
            <stix:Handling>
                <marking:Marking>
                    <marking:Controlled_Structure>//node() | //@*</marking:Controlled_Structure>
                    <marking:Marking_Structure xsi:type='tlpMarking:TLPMarkingStructureType' color="AMBER"/>
                </marking:Marking>
            </stix:Handling>
            <stix:Information_Source>
                <stixCommon:Identity>
                    <stixCommon:Name>Fake</stixCommon:Name>
                </stixCommon:Identity>
                <stixCommon:Time>
                    <cyboxCommon:Produced_Time>2016-03-23T16:45:05+04:00</cyboxCommon:Produced_Time>
                </stixCommon:Time>
            </stix:Information_Source>
        </stix:STIX_Header>
        <stix:Indicators>
            <stix:Indicator id="CFM:Indicator-c2cfe2c4-ebde-548e-bacd-53e3d996c883" timestamp="2016-03-29T19:33:13+01:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Title>Original CRISP Report Document</indicator:Title>
                <indicator:Description>CRISP-16-307.pdf</indicator:Description>
                <indicator:Observable id="CFM:Observable-980d16cc-91d8-51c6-b957-8bfa539a5439">
                    <cybox:Object id="CFM:Object-8c4c2194-54a6-56d2-bd2b-4f5c8b130ab5">
                        <cybox:Properties xsi:type="ArtifactObj:ArtifactObjectType" suspected_malicious="false" type="File" content_type="application/pdf">
                            <ArtifactObj:Packaging>
                                <ArtifactObj:Encoding algorithm="Base64"/>
                            </ArtifactObj:Packaging>
                            <ArtifactObj:Raw_Artifact><![CDATA[JVBERi0xLjMKJcTl8uXrp/Og0MTGCjQgMCBvYmoKPDwgL0xlbmd0aCA1IDAgUiAvRmlsdGVyIC9GbGF0ZURlY29kZSA+PgpzdHJlYW0KeAErVAhUKFQwNAIhUwsLVPRgo=]]></ArtifactObj:Raw_Artifact>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-3e732203-d463-50ba-b6c2-26c11032a204" timestamp="2016-03-29T19:33:13+02:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">IP Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-810a21f8-85aa-5425-ab07-69c94d72b60c">
                    <cybox:Object id="CFM:Object-2b703a03-81ab-5af9-bb53-e7641cbd8e76">
                        <cybox:Properties xsi:type="AddressObj:AddressObjectType" category="ipv4-addr">
                            <AddressObj:Address_Value condition="Equals">10.10.10.10</AddressObj:Address_Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-82b0c3f9-95d4-5ec7-9e09-30b0bf87cfcd" timestamp="2016-03-29T19:33:13+05:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">IP Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-03e0bab9-a13e-55cf-93d4-cf707aeb831d">
                    <cybox:Object id="CFM:Object-9eb91b53-603a-53e5-9228-958e81c24949">
                        <cybox:Properties xsi:type="AddressObj:AddressObjectType" category="ipv4-addr">
                            <AddressObj:Address_Value condition="Equals">13.13.13.13</AddressObj:Address_Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-52c46f7c-cca9-5d2e-9d3b-a3b1744dcf52" timestamp="2016-03-29T19:33:13+06:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">IP Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-691072b3-c6a4-5615-9c78-cb5977864bb5">
                    <cybox:Object id="CFM:Object-44b996e6-7769-54e0-b5c7-bab92fc8791e">
                        <cybox:Properties xsi:type="AddressObj:AddressObjectType" category="ipv4-addr">
                            <AddressObj:Address_Value condition="Equals">12.12.12.12</AddressObj:Address_Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-052c65e0-c667-5e4c-9970-ac9ddd3511b3" timestamp="2016-03-29T19:33:13+07:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">IP Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-1680f55e-8cd8-5d28-8d33-8488ee747581">
                    <cybox:Object id="CFM:Object-4fa49509-2f5c-5c66-a86f-0babf49a0e33">
                        <cybox:Properties xsi:type="AddressObj:AddressObjectType" category="ipv4-addr">
                            <AddressObj:Address_Value condition="Equals">11.11.11.11</AddressObj:Address_Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-1cf2d34d-007a-5a50-b7c1-cce9faf6f968" timestamp="2016-03-29T19:33:13+08:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">IP Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-686ec0d4-59a0-5909-bfbc-eb31c37becd5">
                    <cybox:Object id="CFM:Object-1f397c22-5d3d-50eb-98e9-6acd82fa329c">
                        <cybox:Properties xsi:type="AddressObj:AddressObjectType" category="ipv4-addr">
                            <AddressObj:Address_Value condition="Equals">14.14.14.14</AddressObj:Address_Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-92130d2c-c3e6-5ed9-bcdc-c826c5d2c5b4" timestamp="2016-03-29T19:33:13+07:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-6f106c1e-2ff9-518b-9c36-6edbe59b36e6">
                    <cybox:Object id="CFM:Object-1b6d7388-4c63-5687-9b57-d9dad435bfc4">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:File_Path condition="Equals">D://replacement.exe</FileObj:File_Path>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-1bed1aca-30e1-5ad3-8bee-6c1dfbff157d" timestamp="2016-03-29T19:33:13+06:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-fa7faba7-7dc4-5bb9-a436-31c726039150">
                    <cybox:Object id="CFM:Object-3391be65-9462-5d30-b1e3-7ff813943bff">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:File_Path condition="Equals">/user/strange/object.sh</FileObj:File_Path>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-8b3ac40a-8595-50fe-bea1-fbd1d85cc428" timestamp="2016-03-29T19:33:13+00:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-6be0296d-d22c-5460-bb3f-7626173874bd">
                    <cybox:Object id="CFM:Object-533cd91f-5a13-548d-8bfb-5e0c3741e74e">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:File_Path condition="Equals">C://window32/tst.dat</FileObj:File_Path>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-a9b071be-fa18-5b49-9d15-e487836adb49" timestamp="2016-03-29T19:33:13+00:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-78fb6ef8-b249-5538-8ce7-07addd671a03">
                    <cybox:Object id="CFM:Object-3893bb9a-339d-52bd-84b7-aba7cb19b92b">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:File_Path condition="Equals">webmail.p55.be</FileObj:File_Path>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-2e95d2ac-1b08-5f38-8522-2f4b2ef3686c" timestamp="2016-03-29T19:33:13+00:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">URL Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-5b7cf542-0c1d-5331-966a-9596b2530f10">
                    <cybox:Object id="CFM:Object-9711b917-fff8-5c3e-acee-dee057dbbda7">
                        <cybox:Properties xsi:type="URIObj:URIObjectType" type="URL">
                            <URIObj:Value condition="Equals">bad.domain.be/poor/path</URIObj:Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-5fd6c616-d923-5e70-916d-dca3a2d1ee02" timestamp="2016-03-29T19:33:13+00:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">URL Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-b28b198f-98a4-5126-8724-18cf8adb0fea">
                    <cybox:Object id="CFM:Object-07e004ab-51d6-5546-b7c4-7f2f9f99436e">
                        <cybox:Properties xsi:type="URIObj:URIObjectType" type="URL">
                            <URIObj:Value condition="Equals">fake.site.com/malicious.js</URIObj:Value>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-32d6ca0d-5896-57ff-84c6-8f18a7a70643" timestamp="2016-03-29T19:33:13+00:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">File Hash Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-cb615fd4-12c8-5232-949f-b38fd5ab6a3f">
                    <cybox:Object id="CFM:Object-5a68f003-c73b-590c-95a4-5db828f9eeb0">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:Hashes>
                                <cyboxCommon:Hash>
                                    <cyboxCommon:Type condition="Equals" xsi:type="cyboxVocabs:HashNameVocab-1.0">MD5</cyboxCommon:Type>
                                    <cyboxCommon:Simple_Hash_Value>595f44fec1e92a71d3e9e77456ba80d1</cyboxCommon:Simple_Hash_Value>
                                </cyboxCommon:Hash>
                            </FileObj:Hashes>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
            <stix:Indicator id="CFM:Indicator-1f59ad38-bb9f-5804-8585-9ad6264120a8" timestamp="2016-03-29T19:33:13+00:00" xsi:type='indicator:IndicatorType' version="2.1.1">
                <indicator:Type xsi:type="stixVocabs:IndicatorTypeVocab-1.1">File Hash Watchlist</indicator:Type>
                <indicator:Description>CRISP Report Indicator</indicator:Description>
                <indicator:Observable id="CFM:Observable-eaf5c06f-c0a6-5aaf-8716-a2fda646205d">
                    <cybox:Object id="CFM:Object-b7237e30-b62e-52db-979f-adddff1688e7">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:Hashes>
                                <cyboxCommon:Hash>
                                    <cyboxCommon:Type condition="Equals" xsi:type="cyboxVocabs:HashNameVocab-1.0">MD5</cyboxCommon:Type>
                                    <cyboxCommon:Simple_Hash_Value>627c1bfc48e7a29776d5c36102e13f98</cyboxCommon:Simple_Hash_Value>
                                </cyboxCommon:Hash>
                            </FileObj:Hashes>
                        </cybox:Properties>
                    </cybox:Object>
                </indicator:Observable>
            </stix:Indicator>
        </stix:Indicators>
    </stix:STIX_Package>
    """

STIXRULES = 'test_data/sample_files/stix_rules_example.xml'

SNORTEXAMPLE = 'test_data/sample_files/snort_rules_example.rules'
YARAFILE = 'test_data/sample_files/yara_rules_example.yara'

YARAEXAMPLE = """rule loader_creds_stealer {
meta:
	description = "Detects a malicious loader credential harvester"
	author = "NCCIC Code Analysis Team"
	incident = "10161374"
	date = "2018/02/13"
	hash_0 = "221C6DB5B60049E3F1CDBB6212BE7F41"
	hash_1 = "68970B2CD5430C812BEF5B87C1ADD6EA"
strings:
	$s0 = {85C974228B1185D2741C8B410485C074153D00040000720EB94D5A000033C066390A}
	$s1 = {42004D00500000006630}
	$s2 = {3C004E0055004C004C003E}
	$s3 = "FirstFileExW"
	$s4 = "FindNextFileW"
	$s5 = "SizeofResource"
	$s6 = {2D61636365707465756C61202D642025732025732022257322}
	$s7 = {53656C656374202A2046726F6D2057696E33325F50726F6365737353746F705472616365}
	$s8 = {6400520065006700500072006F00760022005E00290020002600200065006300}
	$s9 = {68006F002E006F005200650067}

condition:
	($s0 and $s1 and $s2 and $s3 and $s4 and $s5) or ($s6 and $s7 and $s8 and $s9)
}"""


class Error(Exception):
    """Base class for exceptions"""
    pass


class ConfigurationError(Error):
    """An exception representing that there was an error in configuring an item"""

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class AuthenticationError(Error):
    """An exception representing that there was an error during an authentication process"""

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)



from .whitelist import Whitelist


class IPv6WL(Whitelist):
    """
    IPv6 address whitelist.
    This class stores its whitelisted IPv6 address in the ipv4addr table.
    """

    @staticmethod
    def getCreateTable():
        return "create table ipv6addr (addr char(40))"

    @staticmethod
    def storeDB(conn, ipv6addrs):
        """Overwrite the addresses in the table with the specified addresses (string) in the database."""
        connection = conn.cursor()
        vals = []
        for ipv6addr in ipv6addrs:
            vals.append((ipv6addr,))
        connection.execute("delete from ipv6addr")
        connection.executemany("insert into ipv6addr(addr) values (?)", iter(vals))
        connection.close()

    def isWhitelisted(self, conn, indicatorType, indicator):
        """
        Return whether or not the indicator of type indicatorType is whitelisted by this whitelist.
        This checks the indicator against the addresses in the table and requires an exact match to be considered
        whitelisted.
        """
        connection = conn.cursor()
        connection.execute("select addr from ipv6addr where addr=?", (indicator,))
        rec = connection.fetchone()
        connection.close
        return rec is not None



import logging
from lqmt.lqm.tool import Tool
from lqmt.lqm.data import AlertAction
from lqmt.tools.to_splunk.splunk_api import ApiHandler
from lqmt.lqm.parsers.FlexTransform.parser import FlexTransformParser
import hashlib


class ToMBL(Tool):
    def __init__(self, config):
        """
        ToMBL tool. Used to push data to MBL using the Splunk Web API.
        :param config: configuration file
        """
        super().__init__(config, [AlertAction.get('All')])
        self._logger = logging.getLogger("LQMT.ToolName.{0}".format(self.getName()))
        self.alerts = {}
        self.sourcetype_lexicon = ['block', 'spearphish', 'malware']

        self._parser = FlexTransformParser({'mbl': 'resources/sampleConfigurations/MBL.cfg'})
        for file_type, source_config in self._config.source_configs.items():
            self._parser.add_parser(file_type, source_config)

            self.splunk_handler = ApiHandler(
                self._config.host,
                self._config.port,
                self._config.username,
                self._config.password,
                cert_check=self._config.cert_check,
                source=self._config.source,
                index=self._config.index,

            )

    def initialize(self):
        super().initialize()

    def process(self, alert, meta):
        """
        Process function. Handles the processing of data for the tool. 
        """
        # parse alert using custom parser
        mbl_alert = self._parser.custom_parser(alert, meta['PayloadFormat'], 'mbl')

        if mbl_alert:
            # pop alert data from list
            mbl_alert = mbl_alert.pop()

            # put alert data into a dictionary. key is generated from a hash of the list
            key_hash = self.compute_hash(str(mbl_alert))
            if key_hash not in self.alerts:
                self.alerts[key_hash] = mbl_alert

    def commit(self):
        """
        Commit function where the data is actually transmitted to Splunk instance. 
        """
        for dict_key, alert_data in self.alerts.items():
            sourcetype, message = self.parse_alert_dictionary(alert_data)

            # sourcetype override from user configuration
            if self._config.sourcetype:
                sourcetype = self._config.sourcetype

            # if sourcetype is valid, then send message
            if sourcetype in self.sourcetype_lexicon:
                self.splunk_handler.send_message(message, sourcetype=sourcetype)

    def cleanup(self):
        pass

    @staticmethod
    def parse_alert_dictionary(alert_data):
        """
        Method used for extracting the source type and message data from the provided alert data dictionary
        :param alert_data: dictionary containing parsed alert data
        :return: returns the sourcetype(str) and formatted message(str) both of which are derived from the alert_data
        dictionary
        """
        message = ""
        sourcetype = ""
        for alert_dictionary in alert_data:
            if 'sourcetype' in alert_dictionary:

                # Extract sourcetype from alert data and delete key from dictionary.
                sourcetype = alert_dictionary['sourcetype'].strip("'")
                del alert_dictionary['sourcetype']

            # format message in key value format. Replace single string with double quotes.
            for key, data in alert_dictionary.items():
                message += "{0}={1} ".format(key, str(data).replace("'", '"'))

        return sourcetype, message

    @staticmethod
    def compute_hash(string):
        """
        Method used for turning an alert string into a hash. Currently being used to sort out duplicate data.
        :param string: Alert data in of the type Str
        :return: returns sha256 digest of given string
        """
        return hashlib.sha256(string.encode('utf-8')).hexdigest()



import logging
from lqmt.lqm.tool import ToolConfig
from lqmt.lqm.exceptions import ConfigurationError


class SysLogConfig(ToolConfig):
    """
       ToSyslog config variables:
        * = required
        *host - address of remote syslog address
        port - port that remote syslog server is listening on(defaults to 514)
        *protocol - protocol you want to communicate with(tcp/udp)
        *messageHead - Define what you want to have at the beginning of every message sent to syslog
        *messageFields - specify what fields you want extracted from the alerts and included in the messages
            note: Selected fields will be appended to end of message as so:
            ex: *messageHead* indicatorType="IPV4" indicator="192.168.1.1"
    """
    def __init__(self, configData, csvToolInfo, unhandledCSV):
        super().__init__(configData, csvToolInfo, unhandledCSV)
        self.logger = logging.getLogger("LQMT.SysLog.{0}".format(self.getName()))

        self.host = self.validation('host', str, required=True)
        self.port = self.validation('port', int, default=514)
        self.protocol = self.validation('protocol', str, default="tcp")
        self.messageHead = self.validation('messageHead', str, required=True)
        self.messageFields = self.validation('messageFields', list, required=True)

        # Additional checks
        if self.protocol.lower() not in ['tcp', 'udp']:
            raise ConfigurationError("The provided value for the variable configuration variable 'protocol' is "
                                     "invalid. Value provided: {0} Valid values: 'tcp', 'udp".format(self.protocol))
